import jazzicon from "@metamask/jazzicon"
import styled from "styled-components"
import { useLayoutEffect, useMemo, useRef } from 'react'

const StyledIdenticon = styled.div<{ $iconSize: number }>`
  height: ${({ $iconSize }) => `${$iconSize}px`};
  width: ${({ $iconSize }) => `${$iconSize}px`};
  border-radius: 50%;
  background-color: ${({ theme }) => theme.surface3};
  font-size: initial;
`

export default function AccountIcon({ address, size }: { address: string, size?: number }) {

    const iconSize = size ?? 24;
    const icon = useMemo(() => jazzicon(iconSize, parseInt(address.slice(2, 10), 16)), [address, iconSize])
    const iconRef = useRef<HTMLDivElement>(null)
    useLayoutEffect(() => {
        const current = iconRef.current
        if (icon) {
            current?.appendChild(icon)
            return () => {
                try {
                    current?.removeChild(icon)
                } catch (e) {
                    console.error('Avatar icon not found')
                }
            }
        }
        return
    }, [icon, iconRef])

    return (
        <StyledIdenticon $iconSize={iconSize}>
            {(
                <span ref={iconRef} />
            )}
        </StyledIdenticon>
    )
}