import eth from "../assets/ETH.png";
import ausd from "../assets/ausd.png";
import btc from "../assets/btc.png";
import xft from "../assets/xft.png"
import { useSelector } from "react-redux";
import { getUtxoBalance, getUtxoStatus } from "../utils/zkhelpers";
import { useState, useEffect } from "react";
import AccountIcon from "./account-icon";
import type { PriceFeed } from '../utils/helpers';
import { getTotal, getSelectedAddress } from '../utils/helpers';
import { UTXOList } from "./activity";
import { Transition } from "@headlessui/react";
import { Transfer } from "./transfer-tab";
import { importDB, exportDB, } from "dexie-export-import";
import { db } from "../utils/db";

export const Account = (props: any) => {

    const [file, setFile] = useState<File>();
    const balances = useSelector((state: any) => state)
    const [total, setTotal] = useState(0)
    const [feed, setFeed] = useState({} as PriceFeed)
    const [showAll, setShowAll] = useState(false)
    const [showOverlay, setShowOverlay] = useState(false)
    const [overlayOpaque, setOverlayOpaque] = useState(false)
    const [sideCard, openSide] = useState("")
    const options = [
        ["ETH", eth],
        ["zkXFT", xft],
        // ["zkETH", eth],
        ["XFT", xft],
        // ["anonUSD", ausd],
        // ["anonBTC", btc],
    ]
    useEffect(() => {
        const load = async () => {
            await setFeed(await getTotal())
            const totals = balances["ETH"] * parseFloat(feed.ETH) + balances["XFT"] * parseFloat(feed.XFT)
            setTotal(totals)
            await getUtxoStatus();
            await getUtxoBalance();
        }
        if (props.isVisible) {
            setTimeout(() => {
                setOverlayOpaque(true)
            }, 150)
            setShowOverlay(true)
        } else {
            setTimeout(() => {
                setShowOverlay(false)
            }, 850)
            setOverlayOpaque(false)
        }
        load()
    }, [props.isVisible])
    const [tab, setTab] = useState("balances")
    const [selectedAsset, setAsset] = useState({ name: "XFT", image: xft })

    const toggleShowAll = () => {
        setShowAll(!showAll)
    }

    const sideToggle = (name, image) => {
        setAsset({ name: name, image: image })
        openSide(name)
    }

    const handleFileChange = (e) => {
        if (e.target.files) {
            setFile(e.target.files[0]);
        }
    };

    const downloadFile = async () => {
        const blob = await exportDB(db,);
        const url = URL.createObjectURL(blob);
        const link = document.createElement('a');
        link.href = url;
        link.download = 'xft-backup.json';
        document.body.appendChild(link);
        link.click();
        document.body.removeChild(link);
        URL.revokeObjectURL(url);
    }

    const restoreFromFile = async () => {
        const db = await importDB(file,);
        db.close()
        alert("utxos recovered successfully")

    }

    return (
        <> 
        {showOverlay &&
            <div id="overlay" className={
                `fixed transition-opacity ease-in-out duration-700 delay-150 z-40 opacity-0
                ${overlayOpaque ?
                    "opacity-100" :
                    "opacity-0"} 
                    fixed w-screen h-screen overflow-hidden inset-0 bg-dark-transparent`} onClick={() => { props.setVisible(false); openSide("") }}></div>}
            <div id="account-show" className={
                `fixed transition-transform duration-300 delay-150 
                ${props.isVisible ?
                    "max-sm:translate-y-0 sm:translate-x-0 ease-in" :
                    "max-sm:translate-y-full sm:translate-x-full ease-in translate-z-0 delay-800"} 
                    bottom-0 z-50 h-[90%] sm:h-screen sm:w-96 border-t-2 sm:border-t-0 sm:border-l-2 
                    sm:right-0 border-purple-transparent bg-back w-screen flex flex-col gap-5`}>
                <div className="relative flex flex-row justify-between m-2">
                    <h1 className="flex w-full pt-2 pl-1 gap-8 justify-items">
                        <a className="flex flex-row group gap-2 bg-purple-transparent rounded-3xl px-1 py-1" target="_blank" href={"//etherscan.io/address/" + balances.address}>
                            <div>
                                <AccountIcon address={balances.address} size={35}></AccountIcon>
                            </div>
                            <p className="text-lg pr-2 self-center transition-all duration-300 ease-in-out group-hover:text-light-font cursor-pointer"> {balances.address.slice(0, 6)}....{balances.address.slice(-4)}</p>
                        </a>
                    </h1>
                    < button className="my-auto bg-purple-transparent transition-colors duration-300 ease-in-out hover:bg-purple rounded-3xl" onClick={() => props.setVisible(false)}>
                        <svg className="fill-white w-10" xmlns="http://www.w3.org/2000/svg" version="1.1" viewBox="2851.016 2041.378 110.968 107.244">
                            <g id="shape-2cd9803c-642d-8016-8003-4ae5b83ebe1c" rx="0" ry="0">
                                <g id="shape-2cd9803c-642d-8016-8003-4ae5b83f16e8" rx="0" ry="0">
                                    <g id="shape-2cd9803c-642d-8016-8003-4ae5b83f16e9" rx="0" ry="0">
                                        <g id="shape-2cd9803c-642d-8016-8003-4ae5b8408a2a">
                                            <g id="fills-2cd9803c-642d-8016-8003-4ae5b8408a2a">
                                                <path className="" rx="0" ry="0" d="M2917.470,2109.862C2918.258,2110.623,2919.405,2110.920,2920.481,2110.642C2921.556,2110.363,2922.397,2109.551,2922.685,2108.512C2922.973,2107.472,2922.665,2106.363,2921.878,2105.602L2910.889,2095.000L2921.878,2084.380C2922.925,2083.183,2922.841,2081.408,2921.686,2080.308C2920.531,2079.207,2918.694,2079.152,2917.470,2080.180L2906.500,2090.800L2895.511,2080.180C2894.275,2079.284,2892.546,2079.403,2891.454,2080.459C2890.361,2081.514,2890.239,2083.186,2891.165,2084.380C2894.828,2087.920,2898.429,2091.460,2902.154,2095.000L2891.165,2105.620C2888.310,2108.320,2892.718,2112.580,2895.511,2109.880L2906.500,2099.242Z">
                                                </path>
                                            </g>
                                        </g>
                                    </g>
                                </g>
                            </g>
                        </svg></button>
                </div>
                <h1 className="text-3xl text-center font-bold top-20 ml-2">${total.toFixed(2)}</h1>
                <a className="text-md underline text-grey" onClick={downloadFile}>Download a backup of your UTXOs</a>
                <div>
                    <p className="text-grey">Recover old utxos from a file backup</p>
                    <input type="file" className="text-sm  file:mr-5 file:py-1 file:px-3 file:border-[1px] file:text-xs file:font-medium hover:file:cursor-pointer hover:file:bg-blue-50 hover:file:text-blue-700" onChange={handleFileChange} />
                    {file !== undefined ?
                    <div className="w-20 h-20 mx-auto">
                        <button className="w-full my-2 mt-2 py-2 bg-button-default hover:bg-border text-sm rounded-lg" onClick={restoreFromFile}>
                            Recover
                        </button>
                    </div> : <></>}
                </div>
                <div className="flex flex-row justify-evenly bg-purple-transparent py-2">
                    <h1 className={`cursor-pointer w-[48%] font-bold text-center text-grey transition-all duration-500 ease-in-out bg-gradient-to-t p-2 rounded-lg bg-size-200 bg-bottom hover:bg-top from-purple-transparent via-purple-transparent to-disabled-button ring-white ring-opacity-50 ring-offset-disabled-button ring-offset-2 hover:text-white  ${tab == "balances" ? "ring-2 text-white" : "text-grey ring-none"}`} onClick={() => setTab("balances")}>Balances</h1>
                    <h1 className={`cursor-pointer w-[48%] font-bold text-center text-grey transition-all duration-500 ease-in-out bg-gradient-to-t p-2 rounded-lg bg-size-200 bg-bottom hover:bg-top from-purple-transparent via-purple-transparent to-disabled-button ring-white ring-opacity-50 ring-offset-disabled-button ring-offset-2 hover:text-white  ${tab == "activity" ? "ring-2 text-white" : "text-grey ring-none"}`} onClick={() => setTab("activity")}>Activity</h1>
                </div>
                {tab == "balances" ?
                    <div className="overflow-y-auto flex flex-col gap-4 ml-2">
                        {/* {sideCard != "" ? <h1 onClick={() => { openSide("") }}>w</h1> : <></>} */}
                        {options.map((r, idx) => {
                            return (
                                <Transition.Root className="ml-4" show={balances[r[0]] > 0 || showAll}>
                                    <Transition.Child
                                        enter={`transition-all ease-in-out duration-500 delay-[200ms]`}
                                        enterFrom="opacity-0 translate-y-6"
                                        enterTo="opacity-100 translate-y-0"
                                        leave="transition-all ease-in-out duration-500"
                                        leaveFrom="opacity-100 translate-y-0"
                                        leaveTo="opacity-0 translate-y-6"
                                    >
                                        {r[0].slice(0, 2) == "zk" ?
                                            <div className="text-lg md:text-3xl flex flex-row gap-5 items-center px-2 py-1 hover:bg-opaque cursor-pointer w-fit rounded-full "
                                                key={idx} onClick={() => sideToggle(r[0], r[1])} >
                                                <img className="h-10" src={r[1]} alt="" />
                                                {balances[r[0]]} {r[0]}
                                            </div> :
                                            <div className="text-lg md:text-3xl flex flex-row gap-5 items-center px-2 py-1  w-fit rounded-full "
                                                key={idx} >
                                                <img className="h-10" src={r[1]} alt="" />
                                                {balances[r[0]]} {r[0]}
                                            </div>
                                        }


                                    </Transition.Child>
                                </Transition.Root>
                            )
                        })}
                        <div className="text-center">{showAll ? <span className="hover:text-light-font cursor-pointer" onClick={toggleShowAll}>Show Less</span> : <span className="text-center hover:text-light-font cursor-pointer" onClick={toggleShowAll}>Show All</span>}</div>
                    </div>
                    :
                    <div className="">
                        <UTXOList></UTXOList>
                    </div>
                }
            </div>
            {/* <Transfer
                isVisible={sideCard}
                setVisible={openSide}
                name={selectedAsset.name}
                image={selectedAsset.image}

            >
            </Transfer> */}
        </>
    )
}