import { useLiveQuery } from 'dexie-react-hooks'
import { db } from '../utils/db'
import eth from "../assets/ETH.png"
import xft from "../assets/xft.png"
import { ZERO_VALUE } from '../../momiji-helpers/utils/types'
import { ActivityCard } from "../components/cards"

export function UTXOList() {

    const getRelativeTime = (timestamp) => {
        let delta = Math.floor((new Date()).getTime() / 1000) - timestamp
        let mappings = ["years", "months", "weeks", "days", "hours", "minutes"]
        let periods = [delta / 31536000, delta / 2592000, delta / 604800, delta / 86400, delta / 3600, delta / 60]
        for (let i = 0; i < periods.length; i++) {
            if (Math.floor(periods[i]) != 0) {
                let period = Math.floor(periods[i])
                let timeUnit = period == 1 ? mappings[i].slice(0, -1) : mappings[i]
                return `${period} ${timeUnit} ago`
            }
        }
        return "A moment ago"
    }

    const getAmount = (amount) => {
        return (amount / BigInt(1e18)).toString()
    }

    const getIcon = (assetType) => {
        if (assetType == ZERO_VALUE) {
            return xft
        }
        return eth
    }
    const getAsset = (assetType) => {
        if (assetType == ZERO_VALUE) {
            return "XFT"
        }
        return "ETH"
    }

    const categorizeUTXOs = (utxos) => {
        let categorizedUTXOs = {
            "queued": [],
            "published": [],
            "spent": []
        }
        utxos.forEach(utxo => {
            if (utxo.status == "queued") {
                categorizedUTXOs.queued.push(utxo)
            } else if (utxo.status == "published") {
                categorizedUTXOs.published.push(utxo)
            } else if (utxo.status == "spent") {
                categorizedUTXOs.spent.push(utxo)
            }
        });
        return categorizedUTXOs
    }

    const utxos = useLiveQuery(() => db.utxo.toArray(), []);
    if (!utxos) return null;

    let categorizedUTXOs = {
        "queued": [],
        "published": [],
        "spent": []
    }
    categorizedUTXOs = categorizeUTXOs(utxos);

    return (
        <>
            {categorizedUTXOs.queued.length > 0 && <h1 className='text-center pb-2 text-purple font-black'>Queued</h1>}
            <ul className="flex flex-col gap-2">
                {categorizedUTXOs.queued?.map((utxo) => (
                    <li key={utxo.id}>
                        <ActivityCard amount={getAmount(utxo.amount)} asset={getAsset(utxo.assetType)} image={getIcon(utxo.assetType)} timestamp={getRelativeTime(utxo.timestamp)} txHash={utxo.txHash} status={utxo.status} chainStatus={utxo.chainStatus}></ActivityCard>
                    </li>
                ))
                }
            </ul>
            {categorizedUTXOs.published.length > 0 && <h1 className='text-center py-2 text-purple font-black'>Published</h1>}
            <ul className="flex flex-col gap-2">
                {categorizedUTXOs.published?.map((utxo) => (
                    <li key={utxo.id}>
                        <ActivityCard amount={getAmount(utxo.amount)} asset={getAsset(utxo.assetType)} image={getIcon(utxo.assetType)} timestamp={getRelativeTime(utxo.timestamp)} txHash={utxo.txHash} status={utxo.status} chainStatus={utxo.chainStatus}></ActivityCard>
                    </li>
                ))
                }
            </ul>
            {categorizedUTXOs.spent.length > 0 && <h1 className='text-center py-2 text-purple font-black'>Spent</h1>}
            <ul className="flex flex-col gap-2">
                {categorizedUTXOs.spent?.map((utxo) => (
                    <li key={utxo.id}>
                        <ActivityCard amount={getAmount(utxo.amount)} asset={getAsset(utxo.assetType)} image={getIcon(utxo.assetType)} timestamp={getRelativeTime(utxo.timestamp)} txHash={utxo.txHash} status={utxo.status}></ActivityCard>
                    </li>
                ))
                }
            </ul >
        </>
    );
}