import { useState } from 'react';

export const Card = (props: any) => {
    return (
        <button className="flex flex-col md:flex-row gap-1 md:gap-14 place-items-center mx-4 md:mx-20 bg-modal rounded-2xl pl-0 md:pl-2 py-2 hover:bg-border" onClick={() => props.onClick([props.title, props.image, props.address])}>
            <img className="right-12 w-12 md:w-20" src={props.image} alt="" />
            <p className="text-md text-center md:text-2xl md:pl-0 md:mr-4 md:justify-self-end">{props.title}</p>
        </button>
    );
}

export const ActivityCard = (props: any) => {
    return (
        <>
            <button className="group relative flex flex-col justify-between place-items-center h-20 transition-all duration-500 ease-in-out hover:h-24 mx-auto px-8 bg-purple-transparent rounded-2xl w-[80%] pl-2 py-2 hover:bg-modal overflow-hidden" onClick={props.hook}>
                <div className="flex items-center w-full justify-between">
                    <img className="w-16 transition-all duration-500 ease-in-out group-hover:rotate-180" src={props.image} alt="" />
                    <p className={`text-center transition-all duration-500 ease-in-out font-bold ${props.status == "spent" ? "text-[grey] line-through" : ""} ${props.status == "published" ? "text-[#97770d] group-hover:text-[gold]" : ""}`}>{props.amount} {props.asset}</p>
                    <p className={`text-center transition-all duration-500 ease-in-out ${props.status == "spent" ? "text-[grey]" : " "} ${props.status == "published" ? "text-[#97770d] group-hover:text-[gold]" : ""}`}>{props.timestamp}</p>
                </div>
                <div className="flex flex-row">
                    {props.chainStatus == "pending" ? <p className="translate-y-20 transition-all duration-500 ease-out group-hover:-translate-y-2">Resubmit</p> : <p className="w-6 h-5"></p>}
                    {props.txHash ? <a className="translate-y-20 transition-all duration-500 ease-in-out group-hover:translate-y-0 absolute bottom-2 right-8 underline hover:text-purple" href={`https://etherscan.io/tx/${props.txHash}`} target="_blank">Etherscan</a> : <></>}
                </div>
            </button>
        </>
    );
}

export const CheckCard = (props: any) => {
    const [isChecked, setChecked] = useState(false);

    const toggleCheck = () => {
        setChecked(!isChecked);
        props.handleIDs(!isChecked, props.value);
    };
    return (
        <button onClick={toggleCheck} className="flex flex-row gap-4 sm:gap-6 place-items-center mx-auto bg-select rounded-2xl pl-2 py-2 h-16 sm:h-18 hover:bg-hover">
            <img className={`${isChecked ? "animate-pulse" :
                "animate-none"} right-12 w-12`} src={props.image} alt="" />
            <p className="text-sm md:text-xl -mr-2 pl-2">{props.title}</p>
            <div className='relative w-12 md:w-20'>
                <svg className='w-10' fill="none" xmlns="http://www.w3.org/2000/svg" version="1.1" viewBox="846.585 5790.097 99.38 99.524" height="99.524">
                    <g id="shape-7d807c81-5df8-80f6-8002-b43a2e0f5165">
                        <g className="fills" id="fills-7d807c81-5df8-80f6-8002-b43a2e0f5165">
                            <path rx="0" ry="0" fill="#8656e2" d="M896.281,5889.620C876.183,5889.622,858.063,5877.500,850.370,5858.905C842.678,5840.311,846.928,5818.907,861.139,5804.675C875.349,5790.442,896.722,5786.184,915.290,5793.885C933.858,5801.587,945.965,5819.732,945.965,5839.859C945.935,5867.327,923.709,5889.587,896.281,5889.620ZZM896.281,5795.112C878.206,5795.110,861.909,5806.013,854.991,5822.736C848.073,5839.459,851.896,5858.709,864.677,5871.509C877.458,5884.308,896.681,5888.137,913.380,5881.209C930.079,5874.281,940.966,5857.961,940.963,5839.859C940.933,5815.159,920.946,5795.143,896.281,5795.112ZZ">
                            </path>
                        </g>

                    </g>

                </svg>
                {isChecked &&
                    <svg className='w-6 absolute top-[2.6rem] left-2 ' fill="none" xmlns="http://www.w3.org/2000/svg" version="1.1" viewBox="881.04 5691.051 35.034 24.594" height="20.594">
                        <g>
                            <g>
                                <path rx="0" ry="0" fill="#8656e2" d="M915.338,5695.345C916.319,5694.362,916.319,5692.770,915.338,5691.788C914.358,5690.806,912.768,5690.806,911.787,5691.788L893.980,5709.621L885.327,5700.955C884.346,5699.973,882.756,5699.973,881.775,5700.955C880.795,5701.937,880.795,5703.529,881.775,5704.511L892.179,5714.930C893.172,5715.883,894.738,5715.883,895.731,5714.930Z">
                                </path>
                            </g>
                        </g>
                    </svg>}
            </div>
        </button>
    );
}

// export const UTXOCard = (props: any) => {
//     return (
//         <button className="flex flex-col md:flex-row gap-1 md:gap-14 place-items-center mx-4 md:mx-20 bg-modal rounded-2xl pl-0 md:pl-2 py-2 hover:bg-border" onClick={() => console.log(props.value)}>
//             <p className="text-md text-center md:text-2xl md:pl-0 md:mr-4 md:justify-self-end">{props.owner}<br />{props.value}</p>
//         </button>
//     );

// }

export const UTXOCard = (props: any) => {
    return (
        <button className="group relative flex flex-col shrink-0 justify-between h-14 items-center transition-all duration-500 ease-in-out mx-auto px-8 bg-purple-transparent rounded-2xl w-40 pl-2 py-2 hover:bg-modal overflow-visible" onClick={() => console.log(props.name)}>
            <div className="flex items-center justify-around w-full my-auto">
                <div className="relative w-10">
                    <img className="absolute -left-1 -top-5 w-10 transition-all duration-500 ease-in-out" src={props.image} alt="" />
                </div>
                <p className=" transition-all duration-500 ease-in-out font-bold">{props.amount}</p>
            </div>

        </button>
    )
}