import { useState, useEffect } from 'react';
import { Card, ActivityCard } from "../components/cards";
import eth from "../assets/ETH.png";
import ausd from "../assets/ausd.png";
import btc from "../assets/btc.png";
import xft from "../assets/xft.png"
import { addresses } from "../utils/constants";
import { UTXO_DB, db } from '../utils/db';
import { ZERO_VALUE } from '../../momiji-helpers/utils/types';
export const Modal = (props: any) => {

    let options
    if (props.transfer) {
        options = [
            ["zkXFT", xft, addresses.XFT_LATEST],
            // ["zkETH", eth, addresses.WETH],
            // ["zkBTC", btc, addresses.anonBTC],
        ]
    }
    else {
        options = [
            // ["ETH", eth, addresses.WETH],
            ["XFT", xft, addresses.XFT_LATEST],
            // ["anonUSD", ausd, addresses.anonUSD],
            // ["anonBTC", btc, addresses.anonBTC],
            ["zkXFT", xft, addresses.XFT_LATEST],
            // ["zkETH", eth, addresses.WETH],
        ]
    }

    const [isVisible, setIsVisible] = useState(false)

    function handleEsc(e: any) {
        if (e.keyCode === 27) {
            props.setSelect(["", null, "0x00"])
        }
    }
    useEffect(() => {
        setTimeout(() => { }, 150);
        setIsVisible(true);
        document.addEventListener("keyup", handleEsc);
        return () => {
            document.removeEventListener("keyup", handleEsc);
        };
    }, [])
    const setSelects = (r: string[]) => {
        if (r[0] != props.selected[0] || props.expert)
            props.setSelect(r)
        else
            props.setSelect(["", null, "0x00"])
    }


    return (
        <>
            <div className={`transition-opacity ease-in duration-300 ${isVisible ? "opacity-100 backdrop-blur-md" : "opacity-0 backdrop-blur-none"}  fixed top-0 left-0 w-screen h-screen z-20 flex items-center justify-center bg-modal-bg  place-items-center text-center`} onClick={() => props.setSelect(["", null, "0x00"])}></div>
            <div className="flex justify-center items-center place-items-center h-[98vh] w-screen">
                <div className="bg-back-transparent border-purple border-2 border-opacity-20 z-40 w-full max-w-[94%] sm:max-w-fit py-4 rounded-xl">
                    <div className="flex flex-col gap-4 z-40">
                        <h3 className="font-bold text-lg pb-4">Select asset</h3>
                        {
                            options.map((r, idx) => {
                                return (
                                    <Card title={r[0]} image={r[1]} address={r[2]} onClick={async () => {
                                        setIsVisible(false)
                                        setSelects(r)
                                    }} key={idx}></Card>
                                );
                            })
                        }
                    </div>
                </div >
            </div >
        </>
    );
}

export const UTXOModal = (props: any) => {
    const [isVisible, setIsVisible] = useState(false)

    const [options, setOptions] = useState([] as UTXO_DB[])

    function handleEsc(e: any) {
        if (e.keyCode === 27) {
            props.setSelect(null)
        }
    }

    const load = async () => {
        let availableUtxos = (await db.utxo.toArray()).filter((utxo) => utxo.status == "published")

        console.log(props.utxos)
        for (let i = 0; i < props.utxos.length; i++) {
            for (let j = 0; j < availableUtxos.length; j++) {
                //check if secrets are equal
                if (props.utxos[i].secret == availableUtxos[j].secret) {
                    availableUtxos.splice(j, 1)
                }
            }
        }

        setOptions(availableUtxos)
    }

    useEffect(() => {
        setTimeout(() => { }, 150);
        setIsVisible(true);
        document.addEventListener("keyup", handleEsc);
        load();
        return () => {
            document.removeEventListener("keyup", handleEsc);
        };
    }, [])

    const getRelativeTime = (timestamp) => {
        let delta = Math.floor((new Date()).getTime() / 1000) - timestamp
        let mappings = ["years", "months", "weeks", "days", "hours", "minutes"]
        let periods = [delta / 31536000, delta / 2592000, delta / 604800, delta / 86400, delta / 3600, delta / 60]
        for (let i = 0; i < periods.length; i++) {
            if (Math.floor(periods[i]) != 0) {
                let period = Math.floor(periods[i])
                let timeUnit = period == 1 ? mappings[i].slice(0, -1) : mappings[i]
                return `${period} ${timeUnit} ago`
            }
        }
        return "A moment ago"
    }

    const getAmount = (amount) => {
        return (amount / BigInt(1e18)).toString()
    }

    const getIcon = (assetType) => {
        if (assetType == ZERO_VALUE) {
            return xft
        }
        return eth
    }
    const getAsset = (assetType) => {
        if (assetType == ZERO_VALUE) {
            return "XFT"
        }
        return "ETH"
    }


    return (
        <>
            <div className={`transition-opacity ease-in duration-300 ${isVisible ? "opacity-100 backdrop-blur-md" : "opacity-0 backdrop-blur-none"}  fixed top-0 left-0 w-screen h-screen z-40 flex items-center justify-center bg-modal-bg  place-items-center text-center`} onClick={() => { props.setSelect(["", null, "0x00"]); props.remove() }}></div>
            <div className="flex justify-center items-center place-items-center h-[98vh] w-screen">
                <div className="bg-back-transparent border-purple border-2 border-opacity-20 z-40 w-full max-w-[94%] sm:max-w-[22rem] py-4 rounded-xl">
                    <div className="flex flex-col gap-4 z-40">
                        {options.length == 0 ? <h3 className="font-bold text-lg pb-4 px-4">No UTXOs available</h3> : <h3 className="font-bold text-lg pb-4">Select UTXO</h3>}
                        {
                            options.map((r) => {
                                return (
                                    <>
                                        <ActivityCard amount={getAmount(r.amount)} asset={getAsset(r.assetType)} image={getIcon(r.assetType)} timestamp={getRelativeTime(r.timestamp)} txHash={r.txHash} status={r.status} chainStatus={r.chainStatus} hook={() => {
                                            setIsVisible(false)
                                            props.setSelect(r)
                                            props.addPublished(r)
                                            console.log(r)
                                        }} />

                                    </>);
                            })
                        }
                    </div>
                </div >
            </div >
        </>
    );
}