import { useEffect, useState } from "react"
import { autoUnstake, getAutoStakes, getManualStakes, ClaimRewardsManual, manualWithdraw } from "../utils/helpers"
import { CheckCard } from "./cards"
import { Button } from "./buttons"
import { toast, ToastContainer } from "react-toastify";
import "react-toastify/dist/ReactToastify.css"
import xft from "../assets/xft.png"

export const StakePositions = (props: any) => {
    const [tokenIDs, setTokenIDs] = useState([] as string[]);
    const [selectedIDS, setSelected] = useState([] as string[]);
    const [isVisible, setIsVisible] = useState(false);

    const load = async () => {
        let res: string[] = [];
        if (props.automatic) {
            res = await getAutoStakes()
        } else {
            console.log("loading..")
            res = await getManualStakes();
        }
        setTokenIDs(res)
    }

    useEffect(() => {
        load()
        setIsVisible(true);
    }, [])

    const handleIDs = (add: Boolean, ID: string) => {
        let selects = selectedIDS
        if (!add) {
            selects = selects.filter(r => r != ID)
            setSelected(selects)
            return
        }
        if (!(selects.includes(ID)) && add) {
            selects.push(ID)
        }
        setSelected(selects)
    }

    const withdraw = async () => {
        for (let i = 0; i < selectedIDS.length; i++) {
            if (props.automatic) {
                await toast.promise(autoUnstake(selectedIDS[i]),
                    {
                        pending: 'Withdrawing in progress',
                        success: 'Withdraw successful!',
                        error: 'Withdraw failed.',
                    })
            } else {
                await manualWithdraw(selectedIDS[i])
            }
        }
        await load()
    }
    const claimRewards = () => {
        for (let i = 0; i < selectedIDS.length; i++) {
            ClaimRewardsManual(selectedIDS[i])
        }
    }

    return (
        <>
            <div className={` transition-opacity  duration-500 delay-150 
            ${isVisible ?
                    "opacity-100 backdrop-blur-md brightness-50" :
                    "opacity-0 backdrop-blur-none brightness-100"} fixed w-screen h-screen min-h-full min-w-full top-0 z-20`} onClick={() => props.enableStakes(false)}>
            </div>
            <ToastContainer
                theme="dark" />
            <div className={
                `transition-all ease-out duration-500 delay-150
            ${isVisible ?
                    "opacity-100" :
                    "opacity-0"} fixed z-40 inset-x-1 bottom-48 grid min-h-[50%] sm:h-[60vh] w-fit place-items-center mx-auto md:w-50`}>

                <div className="relative bg-widget-background max-w-[26.25rem] w-[94vw] sm:w-[80vw] h-fit py-5 p-4 rounded-xl flex flex-col gap-2">
                    <div className={` transition-opacity duration-1000 delay-300
                    ${isVisible ?
                            "opacity-40" : "opacity-0"} absolute bg-border left-0 top-0 max-w-[26.25rem] -z-10 blur-3xl w-[94vw] sm:w-[80vw] h-[102%]`}></div>
                    <div className="text-right pr-6">
                        <button className="w-8 bg-button-default hover:bg-border rounded-lg" onClick={() => props.enableStakes(false)}>
                            <svg className="fill-dark-transparent" xmlns="http://www.w3.org/2000/svg" version="1.1" viewBox="2851.016 2041.378 110.968 107.244">
                                <g id="shape-2cd9803c-642d-8016-8003-4ae5b83ebe1c" rx="0" ry="0">
                                    <g id="shape-2cd9803c-642d-8016-8003-4ae5b83f16e8" rx="0" ry="0">
                                        <g id="shape-2cd9803c-642d-8016-8003-4ae5b83f16e9" rx="0" ry="0">
                                            <g id="shape-2cd9803c-642d-8016-8003-4ae5b8408a2a">
                                                <g id="fills-2cd9803c-642d-8016-8003-4ae5b8408a2a">
                                                    <path className="" rx="0" ry="0" d="M2917.470,2109.862C2918.258,2110.623,2919.405,2110.920,2920.481,2110.642C2921.556,2110.363,2922.397,2109.551,2922.685,2108.512C2922.973,2107.472,2922.665,2106.363,2921.878,2105.602L2910.889,2095.000L2921.878,2084.380C2922.925,2083.183,2922.841,2081.408,2921.686,2080.308C2920.531,2079.207,2918.694,2079.152,2917.470,2080.180L2906.500,2090.800L2895.511,2080.180C2894.275,2079.284,2892.546,2079.403,2891.454,2080.459C2890.361,2081.514,2890.239,2083.186,2891.165,2084.380C2894.828,2087.920,2898.429,2091.460,2902.154,2095.000L2891.165,2105.620C2888.310,2108.320,2892.718,2112.580,2895.511,2109.880L2906.500,2099.242Z">
                                                    </path>
                                                </g>
                                            </g>
                                        </g>
                                    </g>
                                </g>
                            </svg></button>
                    </div>
                    {
                        tokenIDs.map((r: string) => {
                            return <CheckCard title={`Token ID: ${r}`} image={xft} value={r} handleIDs={handleIDs} />
                        })
                    }
                    <div className="mx-4">
                        {props.automatic ? <></> : <Button text={"Claim Rewards"} hook={claimRewards}></Button>}
                        <Button text={"Withdraw"} hook={withdraw}></Button>
                    </div>
                </div>

            </div>

        </>
    )
}