import { useEffect, useState } from 'react';
import { Button, DisableButton } from './buttons';
import { Selector } from './selectors';
import { ethers } from "ethers"

export const Transfer = (props: any) => {

    const [buttonState, setButton] = useState({ "text": "Please enter an amount", "disabled": true })
    const [amountIn, setAmounts] = useState(0)
    const [selectIn, setIn] = useState([props.name, props.image, "0xdeadbeef"])
    const [showOverlay, setShowOverlay] = useState(false)
    const [overlayOpaque, setOverlayOpaque] = useState(false)
    const [recipient, setRecipient] = useState("")

    useEffect(() => {

        if (props.isVisible) {
            setTimeout(() => {
                setOverlayOpaque(true)
            }, 150)
            setShowOverlay(true)
        } else {
            setTimeout(() => {
                setShowOverlay(false)
            }, 850)
            setOverlayOpaque(false)
        }
        if (props.name !== undefined && props.image !== undefined) {
            setIn([props.name, props.image, "0xdeadbeef"])
        }
        setTimeout(() => {
            setAmounts(0)
        }, 700)
    }, [props.isVisible])

    useEffect(() => {
        checkState()
    }, [amountIn, selectIn, recipient])

    const checkState = () => {
        const tokenIn = selectIn[0]
        let state = { ...buttonState }
        state.disabled = true
        if (tokenIn.slice(0, 2) == "zk" && amountIn > 0) {
            if (recipient == "" || !ethers.isAddress(recipient)) {
                state.text = "Please enter a valid address"
                state.disabled = true
            }
            else {
                state.text = "Transfer"
                state.disabled = false
            }
        }
        else if (amountIn <= 0) {
            state.text = "Please enter an amount"
            state.disabled = true
        }
        if (state != buttonState) {
            setButton(state)
        }
    }

    return (
        <>
            {showOverlay &&
                <div id="overlay" className={`fixed transition-opacity ease-in-out duration-700 delay-150 z-50 opacity-0 
                ${overlayOpaque ?
                        "opacity-100" :
                        "opacity-0"} 
                max-sm:w-full sm:w-96 h-screen sm:bottom-0 sm:right-0`} onClick={() => props.setVisible(false)}></div>}
            <div id="transfer" className={`fixed transition-all inset-none max-sm:left-2 right-[23rem] p-2 pr-6 max-sm:top-[19rem] sm:top-48 ease-in-out duration-700 delay-150 z-50 sm:z-40 
                    ${props.isVisible ?
                    "translate-x-0 ease-in-out" :
                    "translate-x-[200%] ease-in-out"}
                     justify-start align-items-center max-sm:w-screen sm:w-96 h-80 bg-purple rounded-xl bg-opacity-70`}>
                <Selector
                    setAmount={setAmounts}
                    amount={amountIn}
                    setSelect={setIn}
                    data={selectIn}
                    noSelect={true}
                ></Selector>
                <div className={`flex flex-row bg-back h-[12vh] md:h-20 mt-2 align-items-center justify-evenly rounded-lg md:rounded-2xl relative`}>
                    <input name="recipient" autoComplete="off" type={"text"} className="absolute outline-none w-[80%] bg-transparent text-sm md:text-xl left-4 top-[40%]" placeholder="Recipient Address" onChange={(r) => setRecipient(r.target.value)} />
                </div>
                {buttonState.disabled ? (
                    <DisableButton text={buttonState.text}></DisableButton>
                ) : (
                    <Button text={buttonState.text}></Button>
                )}
            </div>

        </>
    )

}