import react, { ReactNode } from "react";
import { useState, useEffect } from 'react';
import { Button, DisableButton } from './buttons';
import { recover } from "../utils/wallet_helpers"
import { ActivityCard } from './cards'
import { ExpertToggle } from "./toggles";
import { db } from '../utils/db'
import eth from "../assets/ETH.png"
import xft from "../assets/xft.png"
import { useLiveQuery } from 'dexie-react-hooks';
import { ZERO_VALUE } from "../../momiji-helpers/utils/types";

interface ParentComponentProps {
    children: ReactNode;
}

export const Widget: react.FC<ParentComponentProps> = ({ children }) => {
    return (
        <div className="fixed top-0 left-0 w-screen h-dvh flex items-center justify-center">
            <div className="transition-all duration-500 ease-in-out bg-widget-background w-[94vw] sm:w-[80vw] max-w-[26.25rem] py-5 md:mx-auto border-border p-4 rounded-xl">
                {children}
            </div>
        </div>
    );
};


export const SubBar: react.FC<ParentComponentProps> = ({ children }) => {
    return (
        <div className="grid place-items-center">
            <div className="fixed z-20 text-xs font-bold text-center my-4 text-purple md:text-xl md:absolute md:top-6 bottom-10 bg-mini-widget w-full md:mt-[5vh] md:mx-auto md:min-h-fit border-border p-4 rounded flex flex-row gap-4 justify-around">
                {children}
            </div>
        </div>
    )
}

export const ImportDialog = (props: any) => {

    const [isVisible, setIsVisible] = useState(false)
    const [loading, setLoading] = useState(false)
    const utxos = useLiveQuery(() => db.utxo.toArray(), [])

    const load = async () => {
        while (true) {
            await setTimeout(() => { }, 550)
        }
    }

    const getAmount = (amount) => {
        return amount / BigInt(1e18)
    }

    const getIcon = (assetType) => {
        if (assetType == ZERO_VALUE) {
            return xft
        }
        return eth
    }
    const getAsset = (assetType) => {
        if (assetType == ZERO_VALUE) {
            return "XFT"
        }
        return "ETH"
    }

    useEffect(() => {
        setTimeout(() => { }, 550)
        setIsVisible(true)
    }, [])
    const importUtxos = async () => {
        setLoading(true)
        await recover()
        setLoading(false)
        props.setIsOpen(false)
    }
    return (
        <>
            {props.isOpen ? <>
                <div className={`transition-opacity ease-in duration-300 ${isVisible ? "opacity-100 backdrop-blur-md" : "opacity-0 backdrop-blur-none"}  fixed top-0 left-0 w-screen h-screen z-20 flex items-center justify-center bg-modal-bg  place-items-center text-center`}></div>
                <div className="flex justify-center items-center place-items-center h-[98vh] w-screen">
                    <div className="bg-back-transparent border-purple border-2 border-opacity-20 z-40 w-[94vw] sm:w-[80vw] max-w-[26.25rem] py-5 md:mx-auto rounded-xl">
                        <div className="flex flex-col gap-4 z-40">

                            <h1 className="text-center text-2xl font-bold">{loading ? "Importing UTXOS" : "Getting started"}</h1>
                            {loading ?
                                <div className='flex flex-col w-full justify-center gap-4'>

                                    <svg className="animate-spin-bezier h-5 w-5 mx-auto text-white" xmlns="http://www.w3.org/2000/svg" fill="none" viewBox="0 0 24 24">
                                        <circle className="opacity-25" cx="12" cy="12" r="10" stroke="currentColor" strokeWidth="4"></circle>
                                        <path className="opacity-75" fill="currentColor" d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4zm2 5.291A7.962 7.962 0 014 12H0c0 3.042 1.135 5.824 3 7.938l3-2.647z"></path>
                                    </svg>
                                    {utxos.length >= 1 ?
                                        <>
                                            <h1 className='text-xl'>Most recent fetched UTXO:</h1>
                                            <ActivityCard amount={getAmount(utxos[utxos.length - 1].amount)} asset={getAsset(utxos[utxos.length - 1].assetType)} image={getIcon(utxos[utxos.length - 1].assetType)} timestamp={""} txHash={""} status={utxos[utxos.length - 1].status} chainStatus={utxos[utxos.length - 1].chainStatus}></ActivityCard>
                                        </>
                                        :
                                        <></>
                                    }
                                </div> : <></>}

                            <div className="flex flex-col w-full px-[15%] items-center">
                                {
                                    loading ?
                                        <></>
                                        :
                                        <>
                                            <Button text={"Create a New Wallet"} hook={() => props.setIsOpen(false)}></Button>
                                            <Button text={"Import an existing wallet"} hook={() => importUtxos()}></Button>
                                        </>
                                }
                            </div>
                        </div>
                    </div >
                </div >
            </>
                :
                <></>}
        </>
    );
}

export const WarningDialog = (props: any) => {

    const [isVisible, setIsVisible] = useState(false)
    const [loading, setLoading] = useState(false)

    const load = async () => {
        while (true) {
            await setTimeout(() => { }, 550)
        }
    }

    const checkSentences = [
        { status: false, text: "This is beta software. It has not been audited yet." },
        { status: false, text: "Your notes are not uploaded on-chain yet, and are the only way to access funds. Losing your notes means loss of access." },
        { status: false, text: "The Momiji protocol is a decentralized network. Nobody can recover your lost data/funds." },
        { status: false, text: "While training wheels are enabled, the protocol can be upgraded/modified - even though the protocol is entirely decentralized." },
        { status: false, text: "Any of the above can lead to loss of funds, and I wish to use the software nonetheless." },
        { status: false, text: "Use this software only where it's legal. Your use implies compliance with local laws. We disclaim all liability for any use of this software in violation of any local laws or regulations." },
    ];
    const turnOn = (idx) => {
        checkSentences[idx].status = !checkSentences[idx].status
    }

    const openUp = () => {
        // for (let i = 0; i < checkSentences.length; i++) {
        //     if (checkSentences[i].status == false) {
        //         console.log(checkSentences);
        //         alert("Please accept all terms before proceeding")
        //         return
        //     }
        // }
        props.setIsOpen(true)
    }

    useEffect(() => {
        setTimeout(() => { }, 550)
        setIsVisible(true)
    }, [])


    return (
    <>
        {props.isOpen ? (
        <>
            <div
            className={`transition-opacity ease-in duration-300 ${
                isVisible
                ? "opacity-100 backdrop-blur-md"
                : "opacity-0 backdrop-blur-none"
            }  fixed top-0 left-0 w-screen h-screen z-20 flex items-center justify-center bg-modal-bg  place-items-center text-center`}
            ></div>
            <div className="flex justify-center items-center place-items-center h-[98vh] w-screen">
            <div className="bg-back-transparent border-purple border-2 border-opacity-20 z-40 w-11/12 max-w-2xl mx-auto rounded-xl p-6">
                <div className="flex flex-col gap-4 z-40 mx-auto text-center">
                <h1 className="text-center text-2xl font-bold">Warning</h1>
                <p>This is experimental software please use at your own risk.</p>
                <div className="flex flex-col space-y-4 text-left justify-between">
                    {checkSentences.map((sentence, i) => (
                    <div key={i} className="flex items-start space-x-3">
                        <input
                        type="checkbox"
                        className="mt-1 flex-shrink-0"
                        onChange={() => turnOn(i)}
                        />
                        <p className="text-sm sm:text-base">{sentence.text}</p>
                    </div>
                    ))}
                </div>
                <Button text={"Proceed"} hook={openUp}></Button>
                </div>
            </div>
            </div>
        </>
        ) : null}
    </>
    );
}


export const Settings = (props: any) => {

    return (
        <div className={`relative z-40 w-full h-fit flex justify-end transition-all duration-500 ease-out ${props.visible ? "" : ""}`}>
            <div className={`absolute transition-all duration-500 ease-in-out bg-[#130523] opacity-95 w-[90%] rounded-xl ${props.visible ? "h-52 " : "w-0 h-0 overflow-hidden delay-200"}`}>
                <div className={`pt-2 px-2 justify-end flex flex-col gap-2 transition-opacity duration-500 ease-in-out ${props.visible ? "opacity-100 delay-300" : "opacity-0"}`}>
                    <ExpertToggle expert={props.expert} setExpert={props.setExpert} name={"Expert Mode"} />
                    <div className="flex flex-row gap-2" title="Enter your maximum slippage tolerance for swaping xft for ether on uniswap  ">    
                            <p>Slippage Amount: </p>
                            <input type="number" className="w-8 bg-[#4a1786] rounded" placeholder={props.slippagePercent}  onChange={(e) => parseFloat(props.setSlippagePercent(e.target.value))}/>
                            <p>%</p>
                    </div>
                    <div className="flex flex-row gap-2" title="how much xft you would like to receive upon withdrawal">    
                            <p>Ether Withdrawal: </p>
                            <input type="number"  className="w-8 bg-[#4a1786] rounded" placeholder={props.swapPercent} onChange={(e) => parseFloat(props.setSwapPercent(e.target.value))} />
                            <p>%</p>
                    </div>
                </div>
            </div>
        </div>
    );
};
