import { ethers } from "ethers"
import { useEffect, useState } from "react"
import { useDispatch, useSelector } from "react-redux"
import { toast, ToastContainer } from "react-toastify"
import "react-toastify/dist/ReactToastify.css"
import xft from "../assets/xft.png"
import { AddUTXO, Button, DisableButton, SwapButton } from "../components/buttons"
import { Modal, UTXOModal } from "../components/modal"
import { Navbar } from "../components/navbar"
import { Selector } from "../components/selectors"
import { UTXOCreate, UTXODragnDrop } from "../components/utxos"
import { ImportDialog, WarningDialog, Settings, Widget } from "../components/widgets"
import { set } from "../features/balances"
import { connectWallet } from "../utils/connect-wallet"
import { addresses } from "../utils/constants"
import { hashDB, UTXO_DB } from "../utils/db"
import { getAll } from "../utils/helpers"
import { generateDeposit, generateExpertTx, generateWithdraw, getUtxoBalance, getUtxoStatus, getProverFeeInXFT } from "../utils/zkhelpers"
import { Withdrawal, NoirFr, ZERO_VALUE } from "../../momiji-helpers/utils/types"

export const Shift = () => {

  const [amountIn, setAmountIn] = useState(0)
  const [amountOut, setAmountOut] = useState(0)
  const [recipient, setRecipient] = useState("")
  const [swapPercent, setSwapPercent] = useState(1)
  const [slippagePercent, setSlippagePercent] = useState(1)
  const [feeXFT, setFeeXFT] = useState("0")
  const [selectIn, setIn] = useState(["XFT", xft, addresses.XFT_LATEST])
  const [selectOut, setOut] = useState(["", null, "0x00"])
  const [buttonState, setButton] = useState({ "text": "Please Enter An Amount", "disabled": true, "recipient": "false" })
  const [isVisible, setIsVisible] = useState(false)
  // const [importOpen, setImportOpen] = useState(false);
  const [warningSeen, warningSet] = useState(false)
  const [expertSelect, setExpertSelect] = useState(null as UTXO_DB)
  const [expertIn, setExpertIn] = useState(0)
  const [expertOut, setExpertOut] = useState(0)
  const [expertMode, setExpertMode] = useState(false)
  const [utxos, setUtxos] = useState([
    // { "type": "final", "amount": "100", "image": xft, "name": "XFT" },
  ])
  const [publishedUtxos, setPublished] = useState([] as UTXO_DB[])

  const [utxosOut, setOutUtxos] = useState([
    // { "type": "final", "amount": "100", "image": xft, "name": "XFT", recipient: "0x00" },
  ])

  const addPublished = (utxo: UTXO_DB) => {
    console.log
    let newPublished = publishedUtxos
    newPublished.push(utxo)
    let newUtxos = utxos
    newUtxos[utxos.length - 1].amount = (BigInt(utxo.amount) / BigInt(1e18)).toString()

    newUtxos[utxos.length - 1].type = "published"
    if (utxo.assetType == ZERO_VALUE) {
      newUtxos[utxos.length - 1].image = xft
      newUtxos[utxos.length - 1].name = "zkXFT"
    } else {
      newUtxos[utxos.length - 1].image = xft
      newUtxos[utxos.length - 1].name = "zkETH"
    }
    setUtxos(newUtxos)
    setPublished(newPublished)
  }

  const addEmptyUTXO = (stateUtxo, setterUtxo) => {
    if (stateUtxo.length >= 16) {
      return
    }
    if (stateUtxo.filter((r) => r.type == "input").length >= 1) {
      return
    }
    setExpertOut(0)
    setExpertIn(0)
    setIn(["", null, "0x00"])
    setOut(["", null, "0x00"])
    setExpertSelect(null)
    const newUtxos = stateUtxo
    setAmountIn(0)

    newUtxos.push({ "type": "input", "amount": "0", "image": xft, "name": "xft" })
    setterUtxo(newUtxos)
  }


  // Removes the last UTXO from the list; Called when user clicks out of selecting a ZK UTXO
  const removeLastUTXO = () => {
    let newUtxos = utxos
    newUtxos.pop()
    setUtxos(newUtxos)
  }

  const removeCurrentUTXO = (idx: number) => {
    if (utxos[idx].type == "published") {
      let newPublished = publishedUtxos
      newPublished.pop()
      setPublished(newPublished)
    }
    const newUtxos = utxos
    newUtxos.splice(idx, 1)
    setUtxos(newUtxos)
  }
  const removeOutUTXO = (idx: number) => {
    const newUtxos = utxosOut
    newUtxos.splice(idx, 1)
    setOutUtxos(newUtxos)
  }

  const toggleIsVisible = async () => {
    if (isVisible) {
      await new Promise(r => setTimeout(r, 100));
    }
    setIsVisible(!isVisible)
  }

  const dispatch = useDispatch()
  const load = async () => {
    await connectWallet(() => { })
    const b = await getAll()
    dispatch(set(b))
    getUtxoStatus()
    // const hashes = await hashDB.hash.toArray()
    // if (hashes.length == 0) {
    //   setImportOpen(true)
    // }
    setFeeXFT(await getProverFeeInXFT(700_000n))
  };

  useEffect(() => {
    if (!warningSeen) {
      return
    }
    load()
  }, [warningSeen]);

  useEffect(() => {
    if (!expertMode) {
      checkState()
    }
  }, [amountIn, amountOut, selectIn, selectOut, recipient])
  useEffect(() => {
    const load = async () => {
      if (expertMode) {
        setFeeXFT(await getProverFeeInXFT(3_000_000n))
      } else {
        setFeeXFT(await getProverFeeInXFT(700_000n))
      }
    }
    load()
  }, [expertMode])

  const switchSelects = () => {
    const swap = selectIn
    const amountSwap = amountIn
    setIn(selectOut)
    setAmountIn(amountOut)
    setAmountOut(amountSwap)
    setOut(swap)
  }

  const balances = useSelector((state: any) => { return { "address": state.address } });

  const setAmounts = (amount: number) => {
    setAmountIn(amount)
    setAmountOut(amount)
  }

  const checkState = () => {
    const tokenIn = selectIn[0]
    const tokenOut = selectOut[0]

    let state = { ...buttonState }
    state.disabled = false

    if (tokenIn.slice(0, 2) == "zk" && tokenOut.slice(0, 2) != "zk") {
      state.recipient = "true"
      if (recipient == "" || !ethers.isAddress(recipient)) {
        state.text = "Please enter a valid address"
        state.disabled = true
      }
      else {
        state.text = "Withdraw"
        state.disabled = false
      }
    }
    if (tokenIn.slice(0, 2) != "zk" && tokenOut.slice(0, 2) != "zk") {
      state.text = "Please select at least one zk asset"
      state.disabled = true
    }

    if (tokenIn.slice(0, 2) != "zk" && tokenOut.slice(0, 2) == "zk") {
      state.text = "Deposit"
      state.recipient = "false"
    }

    if (tokenIn.slice(0, 2) == "zk" && tokenOut.slice(0, 2) == "zk") {
      state.text = "Swap"
      state.recipient = "false"
    }

    if (amountIn <= 0) {
      state.text = "Please Enter An Amount"
      state.disabled = true
    }
    if (tokenIn == "" || tokenOut == "") {
      state.text = "Please Select An Asset"
      state.disabled = true
      state.recipient = "false"
    }
    if (state != buttonState) {
      setButton(state)
    }
  }

  const handleShifting = async () => {
    await getUtxoStatus()
    const tokenA = selectIn[0]
    const tokenB = selectOut[0]

    if (tokenA == "" || tokenB == "") {
      return
    }

    const totalIn = await getUtxoBalance().then((result) => {
      return /*BigInt(result) + */ BigInt(Number(amountIn) * 1e18)
    })
    const totalOut = BigInt(Number(amountOut) * 1e18)
    // if (BigInt(totalIn) === BigInt(totalOut)) return Promise.reject("Deposit cannot equal withdraw") // Todo: Use better checking
    if (totalOut  > totalIn) return Promise.reject("Insufficient funds")
    if (totalIn === BigInt(0) && totalOut === BigInt(0)) return Promise.reject("Inputs cannot be zero")

    if (tokenA.slice(0, 2) == "zk" && tokenB.slice(0, 2) != "zk") {
      setButton({ text: "Withdrawing..", disabled: true, recipient: "false" })

      await toast.promise(generateWithdraw(
        {
          amount: (new NoirFr(BigInt(amountOut * 1e18))).toString(),
          recipient: recipient,
          swap_percentage: swapPercent
        } as Withdrawal,
        slippagePercent,
        parseInt(feeXFT)
      ), {
        pending: "Withdrawal in progress...",
        success: "Withdrawal successful",
        error: "Withdrawal failed"
      }).then((res) => { console.log(res) }, (err) => console.error(err))
    }

    if (tokenA.slice(0, 2) != "zk" && tokenB.slice(0, 2) == "zk") {
      setButton({ text: "Depositing", disabled: true, recipient: "false" })
      await toast.promise(generateDeposit(amountIn, parseInt(feeXFT)), {
        pending: "Deposit in progress...",
        success: "Deposit successful",
        error: "Deposit failed"
      }).then((res) => { console.log(res) }, (err) => console.error(err))
    }

    if (tokenA.slice(0, 2) == "zk" && tokenB.slice(0, 2) == "zk") {
      toast.error("Swaps aren't available for the time being sadly :(")
      console.log("Swaps aren't available for the time being sadly :(")
    }
    await getUtxoStatus()
    let state = { ...buttonState }
    state.disabled = false
    setButton(state)
  }

  const handleExpert = async () => {
    let withdrawals: Withdrawal[] = []
    let inputs = []
    let outputs = []
    let publishedIdx = 0

    for (let i = 0; i < utxos.length; i++) {
      let currentUtxo = {}

      if (utxos[i].type == "published") {
        currentUtxo = {
          ...publishedUtxos[publishedIdx],
        }
        publishedIdx++
      } else {
        currentUtxo = {
          assetType: "0x0",
          amount: utxos[i].amount,
        }
      }

      inputs.push(
        currentUtxo
      )
    }
    console.log(utxosOut)
    for (let i = 0; i < utxosOut.length; i++) {

      if (utxosOut[i].name.slice(0, 2) == "zk") {
        outputs.push(
          {
            assetType: ZERO_VALUE,
            amount: utxosOut[i].amount
          })
      } else {
        outputs.push(
          {
            assetType: "0x0",
            amount: utxosOut[i].amount
          }
        )
        withdrawals.push({ amount: (new NoirFr(BigInt(utxosOut[i].amount * 1e18))).toString(), recipient: utxosOut[i].recipient, swap_percentage: swapPercent })
      }
    }

    inputs = (inputs.length > 0) ? inputs : undefined
    outputs = (outputs.length > 0) ? outputs : undefined
    withdrawals = (withdrawals.length > 0) ? withdrawals : undefined

    await toast.promise(generateExpertTx(inputs, outputs, withdrawals, slippagePercent, feeXFT), {
      pending: "Deposit in progress...",
      success: "Deposit successful",
      error: "Deposit failed"
    }).then((res) => { console.log(res) }, (err) => console.error(err))

  }

  return (
    <div className="bg-back min-h-screen text-white text-center">
      <Navbar route={"shift"}></Navbar>
      <ToastContainer
        style={{ zIndex: 39 }}
        theme="dark"
      />
      {expertMode ?
        <div className="flex flex-col fixed top-0 left-0 w-screen h-dvh items-center justify-center">
          <div className="h-[60vh]">
            <div className="flex flex-row bg-widget-background h-[100%] px-4 pt-8 rounded-xl justify-center items-center">
              <div className="flex flex-col h-[95%] gap-10 w-[16rem] py-8 px-2 bg-back rounded-xl overflow-auto no-scrollbar">
                {utxos.map((r, idx) => {
                  if (r.type == "final" || r.type == "published") {
                    return (
                      <div className="flex flex-row">
                        <label className="flex items-center justify-center bg-purple-transparent w-8 h-8 font-semibold my-auto rounded-full">{idx + 1}</label>
                        <UTXODragnDrop image={r.image} amount={r.amount} name={r.name} remove={removeCurrentUTXO} idx={idx} />
                      </div>
                    )
                  } else if (r.type == "input") {
                    return (<>

                      <UTXOCreate
                        setAmount={setExpertIn}
                        amount={expertIn}
                        setSelect={setIn}
                        data={selectIn}
                        disabled={false}
                        out={false}
                        update={() => {
                          const newUtxos = utxos
                          if (selectIn[1] == null || expertIn == 0 || expertIn == null)
                            return
                          newUtxos[idx].amount = expertIn.toString()
                          newUtxos[idx].type = "final"
                          newUtxos[idx].image = selectIn[1]
                          newUtxos[idx].name = selectIn[0]
                          setUtxos(newUtxos)
                        }}
                      />
                    </>
                    )
                  }
                })}
                {utxos.length == 0 || utxos[utxos.length - 1].type != "input" && utxos.length < 16 ? <AddUTXO hook={() => addEmptyUTXO(utxos, setUtxos)} /> : <></>}
              </div>
              <div className="flex z-10 my-auto -mx-[0.67rem]"><SwapButton expert={true} /></div>
              <div className="flex flex-col h-[95%] gap-10 w-[16rem] py-8 px-2 bg-back rounded-xl overflow-auto no-scrollbar">
                {utxosOut.map((r, idx) => {
                  if (r.type == "final") {
                    return (
                      <div className="flex flex-row">
                        <label className="flex items-center justify-center bg-purple-transparent w-8 h-8 font-semibold my-auto rounded-full">{idx + 1}</label>
                        <UTXODragnDrop image={r.image} amount={r.amount} name={r.name} remove={removeOutUTXO} idx={idx} />
                      </div>
                    )
                  } else if (r.type == "input") {
                    return (<>

                      <UTXOCreate
                        setAmount={setExpertOut}
                        amount={expertOut}
                        setSelect={setOut}
                        data={selectOut}
                        disabled={false}
                        out={true}
                        recipient={recipient}
                        setRecipient={setRecipient}
                        update={() => {
                          if (selectOut[1] == null || expertOut == 0 || expertOut == null)
                            return
                          const newUtxos = utxosOut
                          newUtxos[idx].amount = expertOut.toString()
                          newUtxos[idx].type = "final"
                          newUtxos[idx].image = selectOut[1]
                          newUtxos[idx].name = selectOut[0]
                          newUtxos[idx].recipient = recipient
                          setOutUtxos(newUtxos)
                        }}
                      />
                    </>
                    )
                  }
                })}
                {utxosOut.length == 0 || utxosOut[utxosOut.length - 1].type != "input" && utxosOut.length < 16 ? <AddUTXO hook={() => addEmptyUTXO(utxosOut, setOutUtxos)} /> : <></>}
              </div>
              <div className="relative mb-auto h-5 ">
                <div className="absolute flex w-80 -left-8 -top-6" >
                  <button onClick={() => { toggleIsVisible(); }}>
                    <svg xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" width="30" height="30" viewBox="0 0 50 50" className="fill-grey transition-all duration-500 ease-out hover:rotate-90 active:rotate-90">
                      <path d="M47.16,21.221l-5.91-0.966c-0.346-1.186-0.819-2.326-1.411-3.405l3.45-4.917c0.279-0.397,0.231-0.938-0.112-1.282 l-3.889-3.887c-0.347-0.346-0.893-0.391-1.291-0.104l-4.843,3.481c-1.089-0.602-2.239-1.08-3.432-1.427l-1.031-5.886 C28.607,2.35,28.192,2,27.706,2h-5.5c-0.49,0-0.908,0.355-0.987,0.839l-0.956,5.854c-1.2,0.345-2.352,0.818-3.437,1.412l-4.83-3.45 c-0.399-0.285-0.942-0.239-1.289,0.106L6.82,10.648c-0.343,0.343-0.391,0.883-0.112,1.28l3.399,4.863 c-0.605,1.095-1.087,2.254-1.438,3.46l-5.831,0.971c-0.482,0.08-0.836,0.498-0.836,0.986v5.5c0,0.485,0.348,0.9,0.825,0.985 l5.831,1.034c0.349,1.203,0.831,2.362,1.438,3.46l-3.441,4.813c-0.284,0.397-0.239,0.942,0.106,1.289l3.888,3.891 c0.343,0.343,0.884,0.391,1.281,0.112l4.87-3.411c1.093,0.601,2.248,1.078,3.445,1.424l0.976,5.861C21.3,47.647,21.717,48,22.206,48 h5.5c0.485,0,0.9-0.348,0.984-0.825l1.045-5.89c1.199-0.353,2.348-0.833,3.43-1.435l4.905,3.441 c0.398,0.281,0.938,0.232,1.282-0.111l3.888-3.891c0.346-0.347,0.391-0.894,0.104-1.292l-3.498-4.857 c0.593-1.08,1.064-2.222,1.407-3.408l5.918-1.039c0.479-0.084,0.827-0.5,0.827-0.985v-5.5C47.999,21.718,47.644,21.3,47.16,21.221z M25,32c-3.866,0-7-3.134-7-7c0-3.866,3.134-7,7-7s7,3.134,7,7C32,28.866,28.866,32,25,32z"></path>
                    </svg>
                  </button>
                  <Settings
                    visible={isVisible}
                    toggle={toggleIsVisible}
                    expert={expertMode}
                    setExpert={setExpertMode}
                    slippagePercent={slippagePercent}
                    setSlippagePercent={setSlippagePercent}
                    swapPercent={swapPercent}
                    setSwapPercent={setSwapPercent}
                  ></Settings>
                </div>
              </div>

            </div>

          </div>
          <div className="w-64">
            <p className="">
              Prover Fee: {feeXFT} XFT
            </p>
            {utxos.length == 0 ? <DisableButton text={"Select at least one input"}></DisableButton> : <Button text={"Go"} hook={handleExpert} />}
          </div>
        </div>

        : <Widget>
          <div className="flex justify-end pb-2" >
            <button onClick={() => { toggleIsVisible(); }}>
              <svg xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" width="30" height="30" viewBox="0 0 50 50" className="fill-grey transition-all duration-500 ease-out hover:rotate-90 active:rotate-90">
                <path d="M47.16,21.221l-5.91-0.966c-0.346-1.186-0.819-2.326-1.411-3.405l3.45-4.917c0.279-0.397,0.231-0.938-0.112-1.282 l-3.889-3.887c-0.347-0.346-0.893-0.391-1.291-0.104l-4.843,3.481c-1.089-0.602-2.239-1.08-3.432-1.427l-1.031-5.886 C28.607,2.35,28.192,2,27.706,2h-5.5c-0.49,0-0.908,0.355-0.987,0.839l-0.956,5.854c-1.2,0.345-2.352,0.818-3.437,1.412l-4.83-3.45 c-0.399-0.285-0.942-0.239-1.289,0.106L6.82,10.648c-0.343,0.343-0.391,0.883-0.112,1.28l3.399,4.863 c-0.605,1.095-1.087,2.254-1.438,3.46l-5.831,0.971c-0.482,0.08-0.836,0.498-0.836,0.986v5.5c0,0.485,0.348,0.9,0.825,0.985 l5.831,1.034c0.349,1.203,0.831,2.362,1.438,3.46l-3.441,4.813c-0.284,0.397-0.239,0.942,0.106,1.289l3.888,3.891 c0.343,0.343,0.884,0.391,1.281,0.112l4.87-3.411c1.093,0.601,2.248,1.078,3.445,1.424l0.976,5.861C21.3,47.647,21.717,48,22.206,48 h5.5c0.485,0,0.9-0.348,0.984-0.825l1.045-5.89c1.199-0.353,2.348-0.833,3.43-1.435l4.905,3.441 c0.398,0.281,0.938,0.232,1.282-0.111l3.888-3.891c0.346-0.347,0.391-0.894,0.104-1.292l-3.498-4.857 c0.593-1.08,1.064-2.222,1.407-3.408l5.918-1.039c0.479-0.084,0.827-0.5,0.827-0.985v-5.5C47.999,21.718,47.644,21.3,47.16,21.221z M25,32c-3.866,0-7-3.134-7-7c0-3.866,3.134-7,7-7s7,3.134,7,7C32,28.866,28.866,32,25,32z"></path>
              </svg>
            </button>
          </div>
          <Settings
            visible={isVisible}
            toggle={toggleIsVisible}
            expert={expertMode}
            setExpert={setExpertMode}
            slippagePercent={slippagePercent}
            setSlippagePercent={setSlippagePercent}
            swapPercent={swapPercent}
            setSwapPercent={setSwapPercent}></Settings>
          <Selector
            setAmount={setAmounts}
            amount={amountIn}
            setSelect={setIn}
            data={selectIn}
            disabled={false}
            fees={feeXFT}
          ></Selector>
          <div className="my-2"></div>
          <div className="absolute z-10 left-1/2 transform -translate-x-1/2 -translate-y-1/2"><SwapButton switchSelects={switchSelects} /></div>
          <Selector
            setAmount={setAmountOut}
            amount={amountOut}
            setSelect={setOut}
            data={selectOut}
            disabled={true}
            fees={0}
          ></Selector>

          <div className={(buttonState.recipient == "true") ? `flex flex-row bg-back h-[12vh] md:h-20 mt-2 align-items-center justify-evenly rounded-lg md:rounded-2xl relative` : "hidden h-0"}>
            <input name="recipient" type={"text"} autoComplete="off" className="absolute outline-none w-[80%] bg-transparent text-sm md:text-xl left-4 top-[40%]" placeholder="Recipient Address" onChange={(r) => setRecipient(r.target.value)} />
          </div>
          {buttonState.disabled ? (
            <DisableButton text={buttonState.text}></DisableButton>
          ) : (
            <Button text={buttonState.text} hook={handleShifting}></Button>
          )}
          <div className="relative h-8">
            <p className="absolute left-2 py-2 ">Prover Fee: {feeXFT} XFT</p>
          </div>
        </Widget>}
      {/* {importOpen ? <ImportDialog isOpen={importOpen} setIsOpen={setImportOpen} /> : <></>} */}
      {expertMode && selectIn[0].slice(0, 2) == "zk" && expertSelect == null ? <UTXOModal remove={removeLastUTXO} setSelect={setExpertSelect} selected={expertSelect} utxos={publishedUtxos} addPublished={addPublished}></UTXOModal> : <></>}
      {selectIn[2] == "" ? <Modal setSelect={setIn} selected={selectOut} expert={expertMode}></Modal> : <></>}
      {selectOut[2] == "" ? <Modal setSelect={setOut} selected={selectIn} expert={expertMode}></Modal> : <></>}
      {!warningSeen ? <WarningDialog isOpen={warningSet} setIsOpen={warningSet} /> : <></>}
    </div>
  );
};
