import { useState, useEffect } from "react";
import { Navbar } from "../components/navbar";
import { Widget, SubBar } from "../components/widgets";
import { Button, DisableButton } from "../components/buttons";
import { Modal } from "../components/modal";
import { Selector, DisabledSelector } from "../components/selectors";
import { Toggle } from "../components/toggles";
import { useDispatch } from "react-redux";
import { set } from "../features/balances";
import { getAll, requiredStakingETH, autoStake, manualStake, getRewardInfo } from "../utils/helpers";
import { connectWallet } from "../utils/connect-wallet";
import { StakePositions } from "../components/stake-positions";
import eth from "../assets/ETH.png";
import { toast, ToastContainer } from "react-toastify";
import { getManualStakes, getAutoStakes } from "../utils/helpers";
import "react-toastify/dist/ReactToastify.css"

//auto ETH -> (AnonUSD-AnonBTC)
//manual (AnonUSD-AnonBTC) -> ETH tokenID
export const Stake = () => {
    const dispatch = useDispatch();
    useEffect(() => {

        const load = async () => {
            await connectWallet(() => { })
            const b = await getAll()
            dispatch(set(b))
        }
        load()

    }, [])

    const [select1, setSelect] = useState(["ETH", eth, "0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2"]);
    const [select2, setSelect2] = useState(["ETH", eth, "0xC02aaA39b223FE8D0A0e5C4F27eAD9083C756Cc2"]);
    const [amount, setAmount] = useState("");
    const [amount2, setAmount2] = useState("");
    const [automatic, setAuto] = useState(true);
    const [showStakes, enableStakes] = useState(false);
    const [tokenIDs, setTokenIDs] = useState([] as string[]);
    const [rewards, setRewards] = useState(0);

    const load = async () => {
        let res: string[] = [];
        if (automatic) {
            res = await getAutoStakes()
        } else {
            console.log("loading..")
            res = await getManualStakes();
        }
        setTokenIDs(res)
        let totalRewards = await getRewardInfo(tokenIDs)
        setRewards(totalRewards);
    }
    useEffect(() => {
        load()
    }, [])

    const handleAmounts = async (newAmount: any) => {
        setAmount(newAmount as any);
        if (newAmount == "") {
            setAmount2("");
            return;
        }
        const required = await requiredStakingETH(newAmount.toString());
        setAmount2(required);
    }

    return (
        <div className="bg-back min-h-screen text-white text-center">
            <Navbar route={"stake"}></Navbar>
            <ToastContainer
                theme="dark"
            />
            <SubBar>
                <div>Rewards: {rewards} </div>
                <div>{select1[0]}: <a className="underline decoration-dotted underline-offset-2 hover:text-white" target="_blank" href={"//etherscan.io/address/" + select1[2]}>{`${select1[2].slice(0, 6)}...${select1[2].slice(-4)}`}</a></div>
                <div>{select2[0]}: <a className="underline decoration-dotted underline-offset-2 hover:text-white" target="_blank" href={"//etherscan.io/address/" + select2[2]}>{`${select2[2].slice(0, 6)}...${select2[2].slice(-4)}`}</a></div>
            </SubBar>
            <Widget>
                <div className="flex flex-row relative h-16 mb-9">
                    <h1 className="absolute left-0 flex flex-col">
                        <span>Liquidity</span> <span>Staking</span> <span>Incentive</span>
                    </h1>
                    <div className="absolute right-4 flex flex-row gap-2">
                        <Toggle automatic={automatic} setAuto={setAuto} />
                        <label className="inline-block pl-[0.15rem] hover:cursor-pointer"></label>

                    </div>
                    <button>
                        <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 512 512" className="absolute w-10 right-6 -bottom-4 fill-[#a27dddB0] hover:fill-[#FFFFFFB0]" onClick={() => enableStakes(true)}>
                            <path
                                d="M64 32C28.7 32 0 60.7 0 96v320c0 35.3 28.7 64 64 64h384c35.3 0 64-28.7 64-64V192c0-35.3-28.7-64-64-64H80c-8.8 0-16-7.2-16-16s7.2-16 16-16h368c17.7 0 32-14.3 32-32s-14.3-32-32-32H64zm352 240a32 32 0 1 1 0 64 32 32 0 1 1 0-64z"
                            />
                        </svg>
                    </button>
                </div>
                {automatic ? (
                    <>
                        <Selector
                            setAmount={handleAmounts}
                            amount={amount}
                            setSelect={setSelect}
                            data={select1}
                        ></Selector>
                        <div className="my-2"></div>
                        <DisabledSelector
                            setAmount={null}
                            amount={amount2}
                            setSelect={null}
                            data={select2}
                        ></DisabledSelector>
                    </>
                ) : (
                    <>
                        <div className="flex flex-row bg-back h-24 align-items-center justify-evenly rounded-3xl relative">
                            <input
                                type="number"
                                className="absolute w-[92%] bg-transparent outline-none text-xl left-4 top-[40%] placeholder:italic placeholder:"
                                placeholder="Enter Token ID"
                                onChange={(e: any) => {
                                    setAmount(e.target.value);
                                }}
                            />
                        </div>
                    </>
                )}
                {amount == "" ? (
                    automatic ? (
                        <DisableButton text={"Enter an amount"}></DisableButton>
                    ) : (
                        <DisableButton text={"Enter a tokenID"}></DisableButton>
                    )
                ) : (
                    automatic ? (
                        <Button text={"Stake"} hook={() => {
                            toast.promise(
                                autoStake(amount.toString(), amount2), {
                                pending: 'Staking in progress',
                                success: 'Staking successful!',
                                error: 'Staking failed.',
                            })
                        }}></Button>
                    )
                        : (<Button text={"Stake"} hook={() => {
                            toast.promise(
                                manualStake(amount.toString()),
                                {
                                    pending: 'Staking in progress',
                                    success: 'Staking successful!',
                                    error: 'Staking failed.',
                                })
                        }}></Button>)
                )}
            </Widget>
            {select1[2] == "" ? <Modal setSelect={setSelect}></Modal> : <></>}
            {select2[2] == "" ? <Modal setSelect={setSelect2}></Modal> : <></>}
            {showStakes ? <StakePositions enableStakes={enableStakes} automatic={automatic}></StakePositions> : <></>}
        </div>
    );
};
