import { ethers } from "ethers";
import { EthereumProvider } from '@walletconnect/ethereum-provider'
import { getAll } from "./helpers";
export let Signer: ethers.JsonRpcSigner;
export let Provider: ethers.BrowserProvider;

const projectId = "e84f5d9a0daa6b9574b89e9bb7b3178d";
let provider;
(async () => {
    provider = await EthereumProvider.init({
        projectId, // REQUIRED your projectId
        chains: [1], // REQUIRED chain ids
        showQrModal: true, // REQUIRED set to "true" to use @walletconnect/modal
        methods: [
            "call",
            "eth_sendTransaction",
            "eth_sign",
            "personal_sign",
            "eth_signTypedData",
        ], // REQUIRED ethereum methods
        events: ["chainChanged", "accountsChanged"], // REQUIRED ethereum events
    })
})();

export const connectWallet = async (setConnection: any) => {
    if ((window as any).ethereum == null) {
        // If MetaMask is not installed, we use the default provider,
        // which is backed by a variety of third-party services 
        await provider.connect();
        Provider = new ethers.BrowserProvider(provider);
        Signer = await Provider.getSigner();
        setConnection("connected");
        getAll()
    } else {
        // Connect to the MetaMask EIP-1193 object. This is a standard
        // protocol that allows Ethers access to make all read-only
        // requests through MetaMask.
        Provider = new ethers.BrowserProvider((window as any).ethereum);

        Provider.send("eth_requestAccounts", [])
            .then((accounts: any) => {
                console.log("Connected", accounts[0]);
                setConnection("connected");
            })
            .catch((err: any) => {
                console.log("Failed to connect", err);
            });

        // It also provides an opportunity to request access to write
        // operations, which will be performed by the private key
        // that MetaMask manages for the user.
        Signer = await Provider.getSigner();
    }
};
