import { TransactionBuilder } from "../../momiji-helpers/utils/transactionBuilder";
import { ethers } from 'ethers';
// @ts-ignore
import { Buffer } from "buffer"
import { Signer, Provider, connectWallet } from "./connect-wallet";
import { toast } from 'react-toastify';
import * as types from "../../momiji-helpers/utils/types"
import { db, UTXO_DB } from "./db";
import { getSecret, encryptNote } from "./wallet_helpers";
import { getXFTBalance } from "./helpers";

export const addUTXO = async (utxo: UTXO_DB) => {
  try {
    const id = await db.utxo.add(utxo);
  }
  catch (e) {
    console.error(e)
  }
}
// const rpcNode = "nodev2.offshift.io"
const rpcNode = "ws://172.20.135.119:5150"
export const getUtxoBalance = async () => {
  const globalConfig: types.GlobalConfig = {
    signer: Signer,
    provider: Provider
  }
  const transactionBuilder = new TransactionBuilder(globalConfig)
  await transactionBuilder.initializeTransactionBuilder();
  const utxos = (await db.utxo.toArray());
  let utxoBalance = BigInt(0);
  for (let i = 0; i < utxos.length; i++) {
    const utxo = utxos[i];
    if (utxo.status == "published") {
      utxoBalance += BigInt(utxo.amount);
    }
  }
  return utxoBalance.toString()
}

export const getUtxoStatus = async () => {
  const globalConfig: types.GlobalConfig = {
    signer: Signer,
    provider: Provider
  }
  const utxos = await db.utxo.toArray().then(array => array)
  for (let i = 0; i < utxos.length; i++) {
    const utxo = utxos[i]
    const transactionBuilder = new TransactionBuilder(globalConfig)
    await transactionBuilder.initializeTransactionBuilder()
    if (utxo.chainStatus == "pending") {
      const utxoPrevRoot: string = await transactionBuilder.contracts.state.utxoPrevRoots(ethers.zeroPadValue(utxo.commitment, 32))
      if (utxoPrevRoot != types.toFixedHex(0, true)) {
        db.utxo.update(utxo.commitment, { chainStatus: "published", status: "published" })
      }
    }

    if (utxo.status == "published") {
      const nullifier_hash: string = await transactionBuilder.pedersen_left_right(utxo.secret, utxo.secret)
      const spent: boolean = await transactionBuilder.contracts.state.nullifierHashes(ethers.zeroPadValue(nullifier_hash, 32))
      if (spent) {
        db.utxo.update(utxo.commitment, { status: "spent" })
      }
    }
  }
}

export const generateDeposit = async (_amount: number, _proverFee?: number) => {

  console.log("zkHelpers: generateDeposit")
  const globalConfig: types.GlobalConfig = {
    signer: Signer,
    provider: Provider,
    threads: 32
  }
  const transactionBuilder = new TransactionBuilder(globalConfig)
  await transactionBuilder.initializeTransactionBuilder()
  const [priv, pub] = await getSecret()
  const keyHash: string = ethers.keccak256(Uint8Array.from(priv.d.data.concat(priv.e.data)))
  const keyHashModulo: bigint = BigInt(keyHash) % BigInt(types.NoirFr.MODULUS)
  const utxo_new: types.UTXO_Input = {
    secret: (new types.NoirFr(keyHashModulo)).toString(),
    amount: (new types.NoirFr(BigInt(_amount * 1e18))).toString()
  }

  const gas: bigint = 700_000n
  const transactionFeeInXft = await transactionBuilder._getTransactionFeeInXFT(gas)

  // const utxo_prover: types.UTXO_Input = (_proverFee)
  //   ? { amount: (new types.NoirFr(BigInt(_proverFee * 1e18))).toString() }
  //   : { amount: (new types.NoirFr(BigInt(10e18))).toString() }

  const utxo_prover: types.UTXO_Input = { amount: (new types.NoirFr(BigInt(transactionFeeInXft))).toString() }
  const utxo_encrypted: types.UTXO_Encrypted[] = await transactionBuilder._generateUTXOEncrypted([utxo_new, utxo_prover])

  const raw_inputs: types.TransactionInputsRaw = await transactionBuilder._generateRawTransactionInputs(null, utxo_encrypted, null)

  const deposit_amount: bigint = BigInt(raw_inputs.deposit_amount)
  const xft_balance = await getXFTBalance();
  if (deposit_amount >= xft_balance) {
    return Promise.reject("Insufficient xft balance");
  }

  const utxo_commitment: types.UTXO_Commitment[] = await transactionBuilder._generateUTXOCommitments(raw_inputs, [false, true])

  utxo_encrypted.map((utxo, i, _) => {
    if (utxo_commitment[i].spend_in_same_batch === false) {
      utxo.secret_encrypted = types.ZERO_VALUE;
      utxo.amount_encrypted = types.ZERO_VALUE;
      utxo.asset_type_encrypted = types.ZERO_VALUE
      // utxo.secret_encrypted = "0x" + Buffer.from(encryptNote(pub, (utxo.secret.toString()))).toString("hex");
      // utxo.amount_encrypted = "0x" + Buffer.from(encryptNote(pub, ((BigInt(utxo.amount) / BigInt(1e18)).toString()))).toString("hex");
      // utxo.asset_type_encrypted = "0x" + Buffer.from(encryptNote(pub, utxo.asset_type.toString())).toString("hex");
    } else if (utxo_commitment[i].spend_in_same_batch === true) {
      utxo.secret_encrypted = utxo.secret;
      utxo.amount_encrypted = utxo.amount;
      utxo.asset_type_encrypted = utxo.asset_type;
    }
  })


  const transaction: types.Transaction = await transactionBuilder._generateTransactionProof(null, utxo_encrypted, null)

  for (let i = 0; i < utxo_commitment.length; i++) {
    const utxo = utxo_commitment[i]
    if (utxo.commitment !== types.ZERO_VALUE && utxo.spend_in_same_batch === false) {
      await addUTXO({
        commitment: utxo.commitment,
        secret: utxo.secret,
        amount: BigInt(utxo.amount),
        assetType: utxo.asset_type,
        siblings: utxo.siblings,
        timestamp: Math.floor((new Date()).getTime() / 1000),
        status: "pending",
        // txHash?: string, // we don't know yet
        chainStatus: "pending",
      })
    }
  }
  await chrome.runtime.sendMessage("fmkcbjlhabigfofefgicigpppeocfhcc", JSON.stringify(transaction),
    function(response) {
      console.log(response)
    }
  );

  console.log("zkHelpers.ts - generateDeposit(): transaction sent")
}

function generateCombinations(utxos: UTXO_DB[], requiredAmount: bigint): UTXO_DB[] {
  let accumulatedAmount = BigInt(0)
  let accumulatedUtxos: UTXO_DB[] = []
  for (let i = 0; i < utxos.length; i++) {
    if (utxos[i].amount >= requiredAmount) {
      return [utxos[i]]
    }

    accumulatedAmount += BigInt(utxos[i].amount)
    accumulatedUtxos.push(utxos[i])
    if (accumulatedAmount >= requiredAmount) {
      return accumulatedUtxos
    }
  }
}

export const generateWithdraw = async (_withdrawal: types.Withdrawal, slippage_percentage: number, _proverFee?: number) => {
  console.log("zkHelpers: Generate withdrawal")
  const globalConfig: types.GlobalConfig = {
    signer: Signer,
    provider: Provider,
    threads: 32
  }
  const transactionBuilder = new TransactionBuilder(globalConfig)
  await transactionBuilder.initializeTransactionBuilder()

  const _utxoIn = (await db.utxo.toArray()).filter((utxo) => utxo.status == "published").sort((a, b) => Number((a.amount - b.amount)))
  const gas: bigint = 700_000n
  const transactionFeeInXft = await transactionBuilder._getTransactionFeeInXFT(gas)
  let combination = generateCombinations(_utxoIn, BigInt(_withdrawal.amount) + transactionFeeInXft)

  let utxoInAmount = BigInt(0)
  for (let i = 0; i < combination.length; i++) {
    utxoInAmount += BigInt(combination[i].amount)
  }

  const withdrawalBigInt = BigInt(_withdrawal.amount) + transactionFeeInXft
  const [priv, pub] = await getSecret()
  const keyHash: string = ethers.keccak256(Uint8Array.from(priv.d.data.concat(priv.e.data)))
  const keyHashModulo: bigint = BigInt(keyHash) % BigInt(types.NoirFr.MODULUS)
  let utxo_change: types.UTXO_Input = {
    secret: (new types.NoirFr(keyHashModulo)).toString(),
    amount: (new types.NoirFr(BigInt(utxoInAmount - withdrawalBigInt))).toString()
  }

  // const utxo_prover: types.UTXO_Input = (_proverFee)
  //   ? { amount: (new types.NoirFr(BigInt(_proverFee * 1e18))).toString() }
  //   : { amount: (new types.NoirFr(BigInt(10e18))).toString() }

  const utxo_prover: types.UTXO_Input = { amount: (new types.NoirFr(BigInt(transactionFeeInXft))).toString() }
  const utxosIn: types.UTXO_Commitment[] = combination.map((utxo) => { return { secret: utxo.secret, amount: (new types.NoirFr(utxo.amount)).toString(), asset_type: utxo.assetType } })
  const utxo_encrypted: types.UTXO_Encrypted[] = await transactionBuilder._generateUTXOEncrypted([utxo_change, utxo_prover])
  const withdrawals: types.WithdrawalSwap[] = await transactionBuilder._generateWithdrawals([_withdrawal], slippage_percentage)
  const raw_inputs: types.TransactionInputsRaw = await transactionBuilder._generateRawTransactionInputs(utxosIn, utxo_encrypted, withdrawals)

  const deposit_amount: bigint = BigInt(raw_inputs.deposit_amount)
  if (deposit_amount > BigInt(0)) {
    return Promise.reject("Deposit amount cannot be greater than zero on withdrawal")
  }

  const utxo_commitment: types.UTXO_Commitment[] = await transactionBuilder._generateUTXOCommitments(raw_inputs, [false, true])

  utxo_encrypted.map((utxo, i, _) => {
    if (utxo_commitment[i].spend_in_same_batch === false) {
      utxo.secret_encrypted = types.ZERO_VALUE;
      utxo.amount_encrypted = types.ZERO_VALUE;
      utxo.asset_type_encrypted = types.ZERO_VALUE
      // utxo.secret_encrypted = "0x" + Buffer.from(encryptNote(pub, (utxo.secret.toString()))).toString("hex");
      // utxo.amount_encrypted = "0x" + Buffer.from(encryptNote(pub, ((BigInt(utxo.amount) / BigInt(1e18)).toString()))).toString("hex");
      // utxo.asset_type_encrypted = "0x" + Buffer.from(encryptNote(pub, utxo.asset_type.toString())).toString("hex");
    } else if (utxo_commitment[i].spend_in_same_batch === true) {
      utxo.secret_encrypted = utxo.secret;
      utxo.amount_encrypted = utxo.amount;
      utxo.asset_type_encrypted = utxo.asset_type;
    }
  })

  const transaction: types.Transaction = await transactionBuilder._generateTransactionProof(utxosIn, utxo_encrypted, withdrawals)
  for (let i = 0; i < utxo_commitment.length; i++) {
    const utxo = utxo_commitment[i]
    if (utxo.commitment !== types.ZERO_VALUE && utxo.spend_in_same_batch === false) {
      await addUTXO({
        commitment: utxo.commitment,
        secret: utxo.secret,
        amount: BigInt(utxo.amount),
        assetType: utxo.asset_type,
        siblings: utxo.siblings,
        timestamp: Math.floor((new Date()).getTime() / 1000),
        status: "pending",
        // txHash?: string, // we don't know yet
        chainStatus: "pending",
      })
    }
  }
  await chrome.runtime.sendMessage("fmkcbjlhabigfofefgicigpppeocfhcc", JSON.stringify(transaction),
    function(response) {
      console.log(response)
    }
  );
  console.log("zkHelpers.ts - generateWithdrawal(): transaction sent")
}

export async function getProverFeeInXFT(gas: bigint): Promise<string> {
  const config: types.GlobalConfig = {
    provider: Provider
  }
  const transactionBuilder = new TransactionBuilder(config)
  await transactionBuilder.initializeTransactionBuilder()
  const feeInXFT: string = await transactionBuilder._getTransactionFeeInXFT(gas).then(fee => ethers.formatEther(fee))
  return feeInXFT.slice(0, 6)
}


export const generateExpertTx = async (utxosInputs, outputs, _withdrawals: types.Withdrawal[], slippage_percentage: number, _proverFee?) => {
  const globalConfig: types.GlobalConfig = {
    signer: Signer,
    provider: Provider,
    threads: 32
  }
  const transactionBuilder = new TransactionBuilder(globalConfig)
  await transactionBuilder.initializeTransactionBuilder()

  let deposit = BigInt(0)
  let _utxosIn = []
  for (let i = 0; i < utxosInputs.length; i++) {
    if (utxosInputs[i].assetType !== "0x0") {
      _utxosIn.push(utxosInputs[i])
      continue
    }
    deposit += ethers.parseEther(utxosInputs[i].amount)
  }

  const gas: bigint = 3_000_000n
  const transactionFeeInXft = await transactionBuilder._getTransactionFeeInXFT(gas)

  // const utxo_prover: types.UTXO_Input = (_proverFee)
  //   ? { amount: (new types.NoirFr(BigInt(_proverFee * 1e18))).toString() }
  //   : { amount: (new types.NoirFr(BigInt(10e18))).toString() }

  const utxo_prover: types.UTXO_Input = { amount: (new types.NoirFr(BigInt(transactionFeeInXft))).toString() }

  let utxosIn: types.UTXO_Commitment[] = _utxosIn.map((utxo) => {
    return {
      secret: utxo.secret,
      amount: (new types.NoirFr(utxo.amount)).toString(),
      asset_type: utxo.assetType
    }
  });

  let utxosOut: types.UTXO_Input[] = [];

  for (let i = 0; i < outputs.length; i++) {
    let utxo = outputs[i];
    if (utxo.assetType == "0x0") {
      continue
    }
    const [priv, pub] = await getSecret();
    const keyHash: string = ethers.keccak256(Uint8Array.from(priv.d.data.concat(priv.e.data)));
    const keyHashModulo: bigint = BigInt(keyHash) % BigInt(types.NoirFr.MODULUS);
    utxosOut.push({
      amount: (new types.NoirFr(ethers.parseEther(utxo.amount))).toString(),
      secret: (new types.NoirFr(keyHashModulo)).toString(),
      asset_type: types.ZERO_VALUE,
    })
  }

  utxosOut = utxosOut.concat(utxo_prover)
  deposit += BigInt(utxo_prover.amount)
  const xft_balance = await getXFTBalance();
  if (deposit >= xft_balance) {
    return Promise.reject("Insufficient xft balance");
  }
  let spendInSameBatch: boolean[] = Array(utxosOut.length).fill(false)
  spendInSameBatch[spendInSameBatch.length - 1] = true
  const utxo_encrypted: types.UTXO_Encrypted[] = await transactionBuilder._generateUTXOEncrypted(utxosOut)
  const withdrawals: types.WithdrawalSwap[] = (_withdrawals)
    ? await transactionBuilder._generateWithdrawals(_withdrawals, slippage_percentage)
    : undefined
  _utxosIn = (_utxosIn.length > 0)
    ? _utxosIn
    : undefined

  const raw_inputs: types.TransactionInputsRaw = await transactionBuilder._generateRawTransactionInputsExpert(deposit, utxosIn, utxo_encrypted, withdrawals)
  const utxo_commitment: types.UTXO_Commitment[] = await transactionBuilder._generateUTXOCommitments(raw_inputs, spendInSameBatch)

  utxo_encrypted.map((utxo, i, _) => {
    if (utxo_commitment[i].spend_in_same_batch === false) {
      utxo.secret_encrypted = types.ZERO_VALUE;
      utxo.amount_encrypted = types.ZERO_VALUE;
      utxo.asset_type_encrypted = types.ZERO_VALUE
      // utxo.secret_encrypted = "0x" + Buffer.from(encryptNote(pub, (utxo.secret.toString()))).toString("hex");
      // utxo.amount_encrypted = "0x" + Buffer.from(encryptNote(pub, ((BigInt(utxo.amount) / BigInt(1e18)).toString()))).toString("hex");
      // utxo.asset_type_encrypted = "0x" + Buffer.from(encryptNote(pub, utxo.asset_type.toString())).toString("hex");
    } else if (utxo_commitment[i].spend_in_same_batch === true) {
      utxo.secret_encrypted = utxo.secret;
      utxo.amount_encrypted = utxo.amount;
      utxo.asset_type_encrypted = utxo.asset_type;
    }
  })

  const transaction: types.Transaction = await transactionBuilder._generateTransactionProofExpert(deposit, utxosIn, utxo_encrypted, withdrawals)
  for (let i = 0; i < utxo_commitment.length; i++) {
    const utxo = utxo_commitment[i]
    if (utxo.commitment !== types.ZERO_VALUE && utxo.spend_in_same_batch === false) {
      await addUTXO({
        commitment: utxo.commitment,
        secret: utxo.secret,
        amount: BigInt(utxo.amount),
        assetType: utxo.asset_type,
        siblings: utxo.siblings,
        timestamp: Math.floor((new Date()).getTime() / 1000),
        status: "pending",
        // txHash?: string, // we don't know yet
        chainStatus: "pending",
      })
    }
  }
  await chrome.runtime.sendMessage("fmkcbjlhabigfofefgicigpppeocfhcc", JSON.stringify(transaction),
    function(response) {
      console.log(response)
    }
  );
  console.log("zkHelpers.ts - generateExpertTransaction(): transaction sent")



}