use dep::std;
mod structs;
mod hash;

global utxo_depth = 4;
global batch_depth = 4;
global state_depth = 20;
global ZERO_VALUE = 0x016a430aa58685aba1311244a973a3bc358859da86784be51094368e8fb6f720;

// Levels of an empty pedersen Merkle tree with zero_leaf = sha256("Momiji") % Field Modulus
global zero_hashes: [Field; 20] = [
    0x016a430aa58685aba1311244a973a3bc358859da86784be51094368e8fb6f720, 
    0x018d39625c19fa2cfbebdb940a66d7040bb0ef1b59ff6afd92a13a6b9b2d9865, 
    0x096c00ebc0c52478316b6b9fd16d0cd94c5f45bbe45bbfa8c606197c6119d41f, 
    0x2eaefd3bdd1bfbddd8fc5d972ded58617f752b3e88bd17b791e85e7b8eaacb47, 
    0x11d25ff6aa8a431fbce8e8d9a87a2d7986adf38e724fbe47f15752d0931f14d8, 
    0x01e8677aa02546bd7105a7a9fd31c4ef16b69d1bde90f36e97585d7cc31d50e4, 
    0x2520a755a532994c78f35b89938fbc864ec31ec4fc00363f83f9b12f04980c6a, 
    0x21a666842842d5323e51fdda10300c763d6b07e1b797ef3b0bd3690d667445bc, 
    0x1ce681d6f42b70668c369e7f786166e421dc840f0529bbd065d6b02d14ae0fe8, 
    0x1819b90a43ee28f652735708b2ef01428e21b84872bb3c7576de6e35d107c8ed, 
    0x063d7001431110a5949f5946a501fd28f64159f36ab4b80601ca305ac107b3db, 
    0x09aeef7a06f7daf368d797c2e686c7e9884f000de0bd9d8e73392378b0b1be38, 
    0x137121fd5d795eeecc61ec6a6de66abc589d241540139178cd5408b6ccb32a6e, 
    0x01a93f70a1b7482e0b32e76ce49a1e3c1fc2b5cd4789b6af749d78c42791c21a, 
    0x217bf2cc1f1b505a29b162a7889866af2728f5c6708560c9cc5e987b194c9c81, 
    0x1461dae57d7df7e580279637e5d94e0d734b908dc22aec5c52ed00187050a030, 
    0x295933dd65294cbf4b2c64d8a0daa6983d35d1f64d5adc2c44bd9d370086d7dc, 
    0x24650084f0db0fa4e8234fb251ad046b3ddcb7d6f59160b35cc4086e9e196b80, 
    0x15086d0394bd68847e3e36241cc7b3966f96efdd30a37e9e1e85027a6dacbed2, 
    0x1f87a17564f06581c1e1b2d716120990f898893ecc0e076363c56c5e3b56ef7b
    ];

global zero_root: Field = 0x06f93f503e77fcdcacfe622e66adc639b63e8c0083f5cab5d71d461aa4562c92;

#[export]
fn keccak_tx_with_deposit(
    current_root: Field,
    deposit_amount: Field,
    withdrawals: [Field; 16],
    utxo_spendable: [structs::UTXO_Spendable; 16],
    utxo_new: [structs::UTXO_New; 16],
    contract_only_inputs: structs::ContractOnlyInputs
    
) -> Field {
    let withdrawal_amount: Field = withdrawals.reduce(|a,b| a + b);
    let mut commitment_in: [Field; 16] = [ZERO_VALUE; 16];
    let mut commitment_out: [Field; 16] = [ZERO_VALUE; 16];
    let mut utxo_root: Field = 0;
    let empty_utxo = structs::UTXO {owner: 0, amount: ZERO_VALUE, asset_type: 0};
    let mut utxo_out: [structs::UTXO; 16] = [empty_utxo; 16];

    let mut secrets: [Field; 16] = [0; 16];
    let mut oracle: [Field; 16] = [ZERO_VALUE; 16];
    let mut old_root_proof: [Field; 16] = [0; 16];

    let mut utxo_in: [structs::UTXO; 16] = [empty_utxo; 16];
    let mut nullifier_hashes: [Field; 16] = [ZERO_VALUE; 16];
    let empty_merkle_proofs = structs::MerkleProof {
        path_utxo: [0; utxo_depth],
        path_tx: [0; batch_depth],
        path_historic: [0; state_depth],
        index_utxo: 0,
        index_tx: 0,
        index_historic: 0
    };
    let mut merkle_proofs: [structs::MerkleProof; 16] = [empty_merkle_proofs; 16];

    for i in 0..16 {
        if (utxo_spendable[i].amount != ZERO_VALUE) {
            utxo_in[i] = structs::UTXO {owner: utxo_spendable[i].owner(), amount: utxo_spendable[i].amount, asset_type: utxo_spendable[i].asset_type};
            secrets[i] = utxo_spendable[i].secret;
            nullifier_hashes[i] = utxo_spendable[i].nullifier_hash();
            commitment_in[i] = utxo_spendable[i].to_commitment();
            if (utxo_spendable[i].spend_in_same_batch == false) {
                commitment_in[i] = ZERO_VALUE;
                old_root_proof[i] = utxo_spendable[i].old_root_proof;
                merkle_proofs[i] = utxo_spendable[i].merkle_proof;
            }
            
        }

        if (utxo_new[i].secret != 0) {
            commitment_out[i] = utxo_new[i].to_commitment();
            utxo_out[i] = structs::UTXO { owner: utxo_new[i].owner(), amount: utxo_new[i].amount, asset_type: utxo_new[i].asset_type };
        }
    }
    utxo_root = hash::hash_tree_four(commitment_out);
    let public_inputs = structs::PublicInputs {
        current_root: current_root,
        utxo_root: utxo_root,
        deposit_amount: deposit_amount,
        withdrawals: withdrawal_amount,
        commitment_in: commitment_in,
        commitment_out: commitment_out,
        nullifier_hashes: nullifier_hashes,
        contract_only_inputs: contract_only_inputs.as_keccak()
    };

    public_inputs.as_keccak()

}

#[export]
fn keccak_tx_no_deposit(
    current_root: Field,
    deposit_amount: Field,
    withdrawals: [Field; 16],
    utxo_spendable: [structs::UTXO_Spendable; 16],
    utxo_new: [structs::UTXO_New; 16],
    contract_only_inputs: structs::ContractOnlyInputs
    
) -> Field {
    let withdrawal_amount: Field = withdrawals.reduce(|a,b| a + b);
    let mut commitment_in: [Field; 16] = [ZERO_VALUE; 16];
    let mut commitment_out: [Field; 16] = [ZERO_VALUE; 16];
    let mut utxo_root: Field = 0;
    let empty_utxo = structs::UTXO {owner: 0, amount: ZERO_VALUE, asset_type: 0};
    let mut utxo_out: [structs::UTXO; 16] = [empty_utxo; 16];

    let mut secrets: [Field; 16] = [0; 16];
    let mut oracle: [Field; 16] = [ZERO_VALUE; 16];
    let mut old_root_proof: [Field; 16] = [0; 16];

    let mut utxo_in: [structs::UTXO; 16] = [empty_utxo; 16];
    let mut nullifier_hashes: [Field; 16] = [ZERO_VALUE; 16];
    let empty_merkle_proofs = structs::MerkleProof {
        path_utxo: [0; utxo_depth],
        path_tx: [0; batch_depth],
        path_historic: [0; state_depth],
        index_utxo: 0,
        index_tx: 0,
        index_historic: 0
    };
    let mut merkle_proofs: [structs::MerkleProof; 16] = [empty_merkle_proofs; 16];

    for i in 0..16 {
        if (utxo_spendable[i].amount != ZERO_VALUE) {
            utxo_in[i] = structs::UTXO {owner: utxo_spendable[i].owner(), amount: utxo_spendable[i].amount, asset_type: utxo_spendable[i].asset_type};
            secrets[i] = utxo_spendable[i].secret;
            nullifier_hashes[i] = utxo_spendable[i].nullifier_hash();
            commitment_in[i] = utxo_spendable[i].to_commitment();
            if (utxo_spendable[i].spend_in_same_batch == false) {
                commitment_in[i] = ZERO_VALUE;
                old_root_proof[i] = utxo_spendable[i].old_root_proof;
                merkle_proofs[i] = utxo_spendable[i].merkle_proof;
            }
            
        }

        if (utxo_new[i].secret != 0) {
            commitment_out[i] = utxo_new[i].to_commitment();
            utxo_out[i] = structs::UTXO { owner: utxo_new[i].owner(), amount: utxo_new[i].amount, asset_type: utxo_new[i].asset_type };
        }
    }
    utxo_root = hash::hash_tree_four(commitment_out);
    let public_inputs = structs::PublicInputs {
        current_root: current_root,
        utxo_root: utxo_root,
        deposit_amount: deposit_amount,
        withdrawals: withdrawal_amount,
        commitment_in: commitment_in,
        commitment_out: commitment_out,
        nullifier_hashes: nullifier_hashes,
        contract_only_inputs: contract_only_inputs.as_keccak_without_deposit()
    };

    public_inputs.as_keccak()

}

#[export]
fn create_transaction(
    current_root: Field,
    deposit_amount: Field,
    withdrawals: [Field; 16],
    utxo_spendable: [structs::UTXO_Spendable; 16],
    utxo_new: [structs::UTXO_New; 16],
    contract_only_inputs: structs::ContractOnlyInputs

) -> pub structs::TransactionInputs {
    let withdrawal_amount: Field = withdrawals.reduce(|a,b| a + b);
    let mut commitment_in: [Field; 16] = [ZERO_VALUE; 16];
    let mut commitment_out: [Field; 16] = [ZERO_VALUE; 16];
    let mut utxo_root: Field = 0;
    let empty_utxo = structs::UTXO {owner: 0, amount: ZERO_VALUE, asset_type: 0};
    let mut utxo_out: [structs::UTXO; 16] = [empty_utxo; 16];

    let mut secrets: [Field; 16] = [0; 16];
    let mut oracle: [Field; 16] = [ZERO_VALUE; 16];
    let mut old_root_proof: [Field; 16] = [0; 16];

    let mut utxo_in: [structs::UTXO; 16] = [empty_utxo; 16];
    let mut nullifier_hashes: [Field; 16] = [ZERO_VALUE; 16];
    let empty_merkle_proofs = structs::MerkleProof {
        path_utxo: [0; utxo_depth],
        path_tx: [0; batch_depth],
        path_historic: [0; state_depth],
        index_utxo: 0,
        index_tx: 0,
        index_historic: 0
    };
    let mut merkle_proofs: [structs::MerkleProof; 16] = [empty_merkle_proofs; 16];

    for i in 0..16 {
        if (utxo_spendable[i].amount != ZERO_VALUE) {
            utxo_in[i] = structs::UTXO {owner: utxo_spendable[i].owner(), amount: utxo_spendable[i].amount, asset_type: utxo_spendable[i].asset_type};
            secrets[i] = utxo_spendable[i].secret;
            nullifier_hashes[i] = utxo_spendable[i].nullifier_hash();
            commitment_in[i] = utxo_spendable[i].to_commitment();
            if (utxo_spendable[i].spend_in_same_batch == false) {
                commitment_in[i] = ZERO_VALUE;
                old_root_proof[i] = utxo_spendable[i].old_root_proof;
                merkle_proofs[i] = utxo_spendable[i].merkle_proof;
            }
            
        }

        if (utxo_new[i].secret != 0) {
            commitment_out[i] = utxo_new[i].to_commitment();
            utxo_out[i] = structs::UTXO { owner: utxo_new[i].owner(), amount: utxo_new[i].amount, asset_type: utxo_new[i].asset_type };
        }
    }
    utxo_root = hash::hash_tree_four(commitment_out);
    let public_inputs = structs::PublicInputs {
        current_root: current_root,
        utxo_root: utxo_root,
        deposit_amount: deposit_amount,
        withdrawals: withdrawal_amount,
        commitment_in: commitment_in,
        commitment_out: commitment_out,
        nullifier_hashes: nullifier_hashes,
        contract_only_inputs: contract_only_inputs.as_keccak()
    };

    let private_inputs = structs::PrivateInputs {
        oracle: oracle,
        old_root_proof: old_root_proof,
        secrets: secrets,
        utxo_in: utxo_in,
        merkle_proofs: merkle_proofs,
        utxo_out: utxo_out
    };

    let public_inputs_hash: Field = hash::hash_tx(public_inputs);

    let transaction_inputs = structs::TransactionInputs {
        public_inputs_hash: public_inputs_hash,
        public_inputs: public_inputs,
        private_inputs: private_inputs
    };

    transaction_inputs

}

#[export]
fn tx_as_hash(
    tx: structs::PublicInputs
) -> Field {
    tx.as_hash()
}


#[export]
fn rollup_transaction(
    tx_verifier: structs::VerifierTx,
    recursion_verifier: structs::Verifier,
    previous_accumulator: Field,
    tx: structs::PublicInputs
) -> pub structs::RecursionInputs {

    let tx_as_keccak: Field = tx.as_keccak();

    let mut accumulator_preimage: [u8; 128] = [0; 128];
    for i in 0..32 {
        accumulator_preimage[i] = hash::field_to_u8(previous_accumulator)[i];
        accumulator_preimage[i + 32] = hash::field_to_u8(tx_as_keccak)[i];
        accumulator_preimage[i + 64] = hash::field_to_u8(tx_verifier.key_hash)[i];
        accumulator_preimage[i + 96] = hash::field_to_u8(recursion_verifier.key_hash)[i];
    }

    let accumulator: Field = hash::hash_to_field(std::hash::keccak256(accumulator_preimage, accumulator_preimage.len() as u32));

    let recursion_inputs = structs::RecursionInputs {
        accumulator: accumulator,
        tx_verifier: tx_verifier,
        recursion_verifier: recursion_verifier,
        previous_accumulator: previous_accumulator,
        tx: tx
    };

    recursion_inputs
}


#[export]
fn publish_batch(
    accumulator: Field,
    hist_tree_input: structs::HistoricTreeInput,
    tx_verifier: structs::VerifierTx,
    recursion_verifier: structs::Verifier
) -> pub structs::PublishInputs {

    let old_state_root: Field = hist_tree_input.leaf;
    let tx_root: Field = hash::hash_tree_four(hist_tree_input.utxo_roots);
    let batch_oracle: Field = ZERO_VALUE;
    let batch_root: Field = hash::hash([tx_root, batch_oracle]);
    let new_state_root: Field = hash::hash([batch_root, old_state_root]);
    let new_historic_path: [Field; state_depth] = hash::compute_sibling_path(
        hist_tree_input.path, 
        hist_tree_input.leaf, 
        hist_tree_input.index
    );

    let new_historic_root: Field = hash::compute_merkle_root(
        new_state_root,
        hist_tree_input.index + 1,
        new_historic_path
    );

    let hist_tree = structs::HistoricTree {
        root: hist_tree_input.root,
        new_root: new_historic_root,
        leaf: hist_tree_input.leaf,
        index: hist_tree_input.index,
        old_path: hist_tree_input.path,
        new_path: new_historic_path
    };

    let batch = structs::Batch {
        hist_tree: hist_tree,
        old_state_root: old_state_root,
        new_state_root: new_state_root,
        batch_oracle: batch_oracle,
        utxo_roots: hist_tree_input.utxo_roots
    };

    let mut hash_validation: [u8; 832] = [0; 832];
    for i in 0..32 {
        hash_validation[i] = hash::field_to_u8(batch.new_state_root)[i];
        hash_validation[i + 32] = hash::field_to_u8(batch.hist_tree.root)[i];
        hash_validation[i + 64] = hash::field_to_u8(batch.hist_tree.new_root)[i];
        hash_validation[i + 96] = hash::field_to_u8(accumulator)[i];
        hash_validation[i + 128] = hash::field_to_u8(tx_verifier.key_hash)[i];
        hash_validation[i + 160] = hash::field_to_u8(recursion_verifier.key_hash)[i];
        for j in 0..20 {
            hash_validation[i + 192 + (32 * j)] = hash::field_to_u8(batch.hist_tree.new_path[j])[i];
        }
    }

    let pi_contract_hash: Field = hash::hash_to_field(std::hash::keccak256(hash_validation, hash_validation.len() as u32));

    let publish_inputs = structs::PublishInputs {
        pi_contract_hash: pi_contract_hash,
        accumulator: accumulator,
        batch: batch,
        tx_verifier: tx_verifier,
        recursion_verifier: recursion_verifier
    };
    publish_inputs

}