import setGlobalVars from 'indexeddbshim';
if (typeof global === 'object') setGlobalVars();
import Dexie, { Table } from 'dexie';

export interface UTXO_DB {
    id?: number;
    commitment: string;
    secret: string;
    amount: bigint;
    assetType: string;
    siblings?: string[];
    timestamp: number;
    status: string;
    txHash?: string;
    chainStatus: string;
}

export interface Hash {
    id?: number;
    hash: string;
}

export class UTXOStorage extends Dexie {

    utxo!: Table<UTXO_DB>;

    constructor() {
        super('UTXOStorage');
        this.version(1).stores({
            utxo: '++commitment, secret, amount, assetType, siblings, timestamp, status, chainStatus'
        });
        this.utxo = this.table('utxo');
    }
}

export class HashDB extends Dexie {

    hash!: Table<Hash>;

    constructor() {
        super('HashDB');
        this.version(1).stores({
            hash: '++id, hash'
        })
        this.hash = this.table('hash');
    }
}

export const db = new UTXOStorage();

export const hashDB = new HashDB();
